#include "testlib.h"
#include <iostream>
#include <set>

using namespace std;

vector<int> randomRadiuses(int n, int min_radius, int max_radius, int bias = 0) {
	set<int> L1;
		while (L1.size() != n) {
			int val = rnd.wnext(min_radius, max_radius, bias);
			L1.insert(val);
		}
	vector<int> ret;
	ret.reserve(n);
	for (set<int>::iterator it=L1.begin(); it != L1.end(); ++it) {
		ret.push_back(*it);
	}
	
	return ret;
}

vector<int> randomLayers(int n, int layers, int max_radius, int bias) {
	vector<int> ret;
	ret.reserve(n);
	int layer_size = n / layers + 1;
	for (int layer = 0; layer < layers; ++layer) {
		int min_rad = max_radius / layers * layer + 1;
		int max_rad = (max_radius / layers) * (layer + 1);
		if (n < layer_size*2) {
			layer_size = n;
			max_rad = max_radius;
		}
		vector<int> cret = randomRadiuses(layer_size, min_rad, max_rad, bias);
		ret.insert(ret.end(), cret.begin(), cret.end());
		n -= layer_size;
		if (!n) break;
	}
	return ret;
}

bool is_square(long long x) {
	long long base = (long long) sqrt(x);
	for (long long dx = -5; dx < 6; ++dx) {
		long long candidate = base + dx;
		if (candidate < 0) continue;
		if (candidate*candidate == x) return true;
	}
	return false;
}

pair<long long, long long> far_away_point(long long mindist) {
	double angle = rnd.next(M_PI * 2);
	long long dx = (long long)(cos(angle)*(mindist+1));
	long long dy = (long long)(sin(angle)*(mindist+1));
	return make_pair(dx, dy);
}

int main(int argc, char* argv[]) {
	setName("cococoncentric problem's generator with nonintegral dist");
	registerGen(argc, argv, 1);
	int n = atoi(argv[1]);
	int m = atoi(argv[2]);
	int max_coords = atoi(argv[3]);
	int max_radius = atoi(argv[4]);
	int distance = atoi(argv[5]);
	int layers = atoi(argv[6]);
	int bias = atoi(argv[7]);

	long long x1 = rnd.next(-max_coords, +max_coords);
	long long y1 = rnd.next(-max_coords, +max_coords);

	long long x2, y2;
	while (true) {
		pair<long long, long long> vec = far_away_point(distance);
		x2 = x1 + vec.first;
		y2 = y1 + vec.second;
		if (is_square((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2))) continue;
		if ((x1 == y1) && (x2 == y2)) continue;
		if ((abs(x2) > max_coords) || (abs(y2) > max_coords)) continue;
		break;
	}
	
	vector<int> L1 = randomLayers(n, layers, max_radius, bias);
	vector<int> L2 = randomLayers(m, layers, max_radius, bias);

	printf("%lld %lld\n%d", x1, y1, n);
	for (vector<int>::iterator it=L1.begin(); it != L1.end(); ++it) printf(" %d", *it);
	printf("\n");
	printf("%lld %lld\n%d", x2, y2, m);
	for (vector<int>::iterator it=L2.begin(); it != L2.end(); ++it) printf(" %d", *it);
	printf("\n");

	return 0;
}

