#include "testlib.h"
#include <iostream>
#include <set>

using namespace std;

vector<int> randomRadiuses(int n, int max_radius) {
	set<int> L1;
	if (max_radius > 2*n) {
		while (L1.size() != n) {
			int val = rnd.next(1, max_radius);
			L1.insert(val);
		}
	} else {
		vector<int> pool(max_radius);
		for (int i = 0; i < max_radius; ++i) pool[i] = i + 1;
		shuffle(pool.begin(), pool.end());
		for (int i = 0; i < n; ++i) L1.insert(pool[i]);
	}
	vector<int> ret;
	ret.reserve(n);
	for (set<int>::iterator it=L1.begin(); it != L1.end(); ++it) {
		ret.push_back(*it);
	}
	
	return ret;
}

bool is_square(long long x) {
	long long base = (long long) sqrt(x);
	for (long long dx = -5; dx < 6; ++dx) {
		long long candidate = base + dx;
		if (candidate < 0) continue;
		if (candidate*candidate == x) return true;
	}
	return false;
}

pair<long long, long long> far_away_point(long long dist) {
	const vector<vector<int>> pytha {
		{  1,  0,  1},
		{  3,  4,  5}, {  5, 12, 13}, {  8, 15, 17}, {  7, 24, 25},
		{ 20, 21, 29}, { 12, 35, 37}, {  9, 40, 41}, { 28, 45, 53},
		{ 11, 60, 61}, { 16, 63, 65}, { 33, 56, 65}, { 48, 55, 73},
		{ 13, 84, 85}, { 36, 77, 85}, { 39, 80, 89}, { 65, 72, 97},
		{ 20, 99,101}, { 60, 91,109}, { 15,112,113}, { 44,117,125},
		{ 88,105,137}, { 17,144,145}, { 24,143,145}, { 51,140,149},
		{ 85,132,157}, {119,120,169}, { 52,165,173}, { 19,180,181},
		{ 57,176,185}, {104,153,185}, { 95,168,193}, { 28,195,197},
		{ 84,187,205}, {133,156,205}, { 21,220,221}, {140,171,221},
		{ 60,221,229}, {105,208,233}, {120,209,241}, { 32,255,257},
		{ 23,264,265}, { 96,247,265}, { 69,260,269}, {115,252,277},
		{160,231,281}, {161,240,289}, { 68,285,293}
	};
	vector<vector<int>> interested;
	for (int i = 0; i < pytha.size(); ++i) {
		if (dist % pytha[i][2]) continue;
		interested.push_back(pytha[i]);
	}
	vector<int> ret = rnd.any(interested.begin(), interested.end());
	long long mult = dist / ret[2];
	if (rnd.next(2)) {
		swap(ret[0], ret[1]);
	}
	if (rnd.next(2)) {
		ret[0] *= -1;
	}
	if (rnd.next(2)) {
		ret[1] *= -1;
	}
	return make_pair(ret[0] * mult, ret[1] * mult);
}

int main(int argc, char* argv[]) {
	setName("cococoncentric problem's generator with precision dist");
	registerGen(argc, argv, 1);
	int n = atoi(argv[1]);
	int m = atoi(argv[2]);
	int max_coords = atoi(argv[3]);
	int max_radius = atoi(argv[4]);
	int distance = atoi(argv[5]);

	long long x1 = rnd.next(-max_coords, +max_coords);
	long long y1 = rnd.next(-max_coords, +max_coords);

	long long x2, y2;
	while (true) {
		pair<long long, long long> vec = far_away_point(distance);
		x2 = x1 + vec.first;
		y2 = y1 + vec.second;
		if ((abs(x2) > max_coords) || (abs(y2) > max_coords)) continue;
		break;
	}
	
	vector<int> L1 = randomRadiuses(n, max_radius);
	vector<int> L2 = randomRadiuses(m, max_radius);

	printf("%lld %lld\n%d", x1, y1, n);
	for (vector<int>::iterator it=L1.begin(); it != L1.end(); ++it) printf(" %d", *it);
	printf("\n");
	printf("%lld %lld\n%d", x2, y2, m);
	for (vector<int>::iterator it=L2.begin(); it != L2.end(); ++it) printf(" %d", *it);
	printf("\n");

	return 0;
}

