#include "testlib.h"
#include <iostream>
#include <set>

using namespace std;

vector<int> randomRadiuses(int n, int max_radius) {
	set<int> L1;
	if (max_radius > 2*n) {
		while (L1.size() != n) {
			int val = rnd.next(1, max_radius);
			L1.insert(val);
		}
	} else {
		vector<int> pool(max_radius);
		for (int i = 0; i < max_radius; ++i) pool[i] = i + 1;
		shuffle(pool.begin(), pool.end());
		for (int i = 0; i < n; ++i) L1.insert(pool[i]);
	}
	vector<int> ret;
	ret.reserve(n);
	for (set<int>::iterator it=L1.begin(); it != L1.end(); ++it) {
		ret.push_back(*it);
	}
	
	return ret;
}

bool is_square(long long x) {
	long long base = (long long) sqrt(x);
	for (long long dx = -5; dx < 6; ++dx) {
		long long candidate = base + dx;
		if (candidate < 0) continue;
		if (candidate*candidate == x) return true;
	}
	return false;
}

int main(int argc, char* argv[]) {
	setName("cococoncentric problem's generator with nonintegral dist");
	registerGen(argc, argv, 1);
	int n = atoi(argv[1]);
	int m = atoi(argv[2]);
	int max_coords = atoi(argv[3]);
	int max_radius = atoi(argv[4]);

	long long x1 = rnd.next(-max_coords, +max_coords);
	long long y1 = rnd.next(-max_coords, +max_coords);

	long long x2, y2;
	while (true) {
		x2 = rnd.next(-max_coords, +max_coords);
		y2 = rnd.next(-max_coords, +max_coords);
		if (!is_square((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2))) break;
	}
	
	vector<int> L1 = randomRadiuses(n, max_radius);
	vector<int> L2 = randomRadiuses(m, max_radius);

	printf("%lld %lld\n%d", x1, y1, n);
	for (vector<int>::iterator it=L1.begin(); it != L1.end(); ++it) printf(" %d", *it);
	printf("\n");
	printf("%lld %lld\n%d", x2, y2, m);
	for (vector<int>::iterator it=L2.begin(); it != L2.end(); ++it) printf(" %d", *it);
	printf("\n");

	return 0;
}

