#include "testlib.h"
#include <iostream>
#include <set>

using namespace std;

vector<int> randomRadiuses(int n, int max_radius) {
	set<int> L1;
	if (max_radius > 2*n) {
		while (L1.size() != n) {
			int val = rnd.next(1, max_radius);
			L1.insert(val);
		}
	} else {
		vector<int> pool(max_radius);
		for (int i = 0; i < max_radius; ++i) pool[i] = i + 1;
		shuffle(pool.begin(), pool.end());
		for (int i = 0; i < n; ++i) L1.insert(pool[i]);
	}
	vector<int> ret;
	ret.reserve(n);
	for (set<int>::iterator it=L1.begin(); it != L1.end(); ++it) {
		ret.push_back(*it);
	}
	
	return ret;
}

bool is_square(long long x) {
	long long base = (long long) sqrt(x);
	for (long long dx = -5; dx < 6; ++dx) {
		long long candidate = base + dx;
		if (candidate < 0) continue;
		if (candidate*candidate == x) return true;
	}
	return false;
}

pair<long long, pair<long long, long long> > far_away_point(long long pytha_index, long long dimension_limit) {
	const vector<vector<int>> pytha {
		{  1,  0,  1},
		{  3,  4,  5}, {  5, 12, 13}, {  8, 15, 17}, {  7, 24, 25},
		{ 20, 21, 29}, { 12, 35, 37}, {  9, 40, 41}, { 28, 45, 53},
		{ 11, 60, 61}, { 16, 63, 65}, { 33, 56, 65}, { 48, 55, 73},
		{ 13, 84, 85}, { 36, 77, 85}, { 39, 80, 89}, { 65, 72, 97},
		{ 20, 99,101}, { 60, 91,109}, { 15,112,113}, { 44,117,125},
		{ 88,105,137}, { 17,144,145}, { 24,143,145}, { 51,140,149},
		{ 85,132,157}, {119,120,169}, { 52,165,173}, { 19,180,181},
		{ 57,176,185}, {104,153,185}, { 95,168,193}, { 28,195,197},
		{ 84,187,205}, {133,156,205}, { 21,220,221}, {140,171,221},
		{ 60,221,229}, {105,208,233}, {120,209,241}, { 32,255,257},
		{ 23,264,265}, { 96,247,265}, { 69,260,269}, {115,252,277},
		{160,231,281}, {161,240,289}, { 68,285,293}
	};
	vector<int> ret = pytha[pytha_index];
	long long multx = dimension_limit / ret[0];
	long long multy = multx + 1;
	if (ret[1]) multy = dimension_limit / ret[1];
	long long mult = min(multx, multy);
	long long pretended = ret[2] * mult;
	long long nx = mult * ret[0];
	long long ny = mult * ret[1];
	if (rnd.next(2))
		ny -= 1;
	else
		ny += 1;
	return make_pair(pretended, make_pair(nx, ny));
}

int main(int argc, char* argv[]) {
	setName("cococoncentric problem's generator with precision dist");
	registerGen(argc, argv, 1);
	int n = atoi(argv[1]);
	int m = atoi(argv[2]);
	int max_coords = atoi(argv[3]);
	int max_radius = atoi(argv[4]);
	int pytha_index = atoi(argv[5]);

	long long x1 = -max_coords;
	long long y1 = -max_coords;

	long long x2, y2;
	long long pretender;
	while (true) {
		pair<long long, pair<long long, long long> > vec = far_away_point(pytha_index, 2*max_coords);
		x2 = x1 + vec.second.first;
		y2 = y1 + vec.second.second;
		pretender = vec.first;
		if ((abs(x2) > max_coords) || (abs(y2) > max_coords)) continue;
		if (is_square((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2))) continue;
		break;
	}
	
	vector<int> L1 = randomRadiuses(n, max_radius);
	set<int> mL2;

	for (int i = 0; i < L1.size(); ++i) {
		if (mL2.size() == m) break;
		long long candidate;
		if (L1[i] < pretender) {
			candidate = pretender - L1[i];
		} else {
			candidate = pretender + L1[i];
		}
		if (candidate > max_radius) continue;
		mL2.insert(candidate);
	}

	while (mL2.size() != m) {
		int candidate = rnd.next(1, max_radius);
		mL2.insert(candidate);
	}

	vector<int> L2;
	L2.assign(mL2.begin(), mL2.end());


	printf("%lld %lld\n%d", x1, y1, n);
	for (vector<int>::iterator it=L1.begin(); it != L1.end(); ++it) printf(" %d", *it);
	printf("\n");
	printf("%lld %lld\n%d", x2, y2, m);
	for (vector<int>::iterator it=L2.begin(); it != L2.end(); ++it) printf(" %d", *it);
	printf("\n");

	return 0;
}

