#include <iostream>
#include "testlib.h"
#include <vector>
#include <algorithm>

using namespace std;

long long min_coords, max_coords;
int max_nm;
long long max_radius;
bool deny_overlaps;
bool deny_kisses;


void readRadiuses(int& n, vector<long long> &L, string name) {
	n = inf.readInt(1, max_nm, format("|%s|", name));
	inf.readSpace();
	L.reserve(n);
	for (int i = 0; i < n; ++i) {
		L.push_back(inf.readInt(1, max_radius, format("%s[%d]", name, i).c_str()));
		if (i) {
			ensuref(
				L[i-1] < L[i],
				"%s is not strictly increasing: %d >= %d",
				name, L[i-1], L[i]
			);
		}
		if (i < n - 1) inf.readSpace();
	}
	inf.readEoln();
}

struct Point {
	int family;
	long long coord;
	static long long dist2;
	bool operator<(const Point &other) const {
		if (family == other.family) return coord < other.coord;
		if (family == 0) {
			// coord < dist + other.coord
			// coord - other.coord < dist
			return (coord < other.coord) || ((coord - other.coord)*(coord - other.coord) < dist2);
		} else {
			return !(other < (*this)) && !(other == (*this));
		}
	}
	bool operator==(const Point &other) const {
		if (family == other.family) return coord == other.coord;
		if (family == 0) {
			return (coord - other.coord)*(coord - other.coord) == dist2;
		} else {
			return other == (*this);
		}
	}
	bool operator!=(const Point &other) const {
		return !((*this) == other);
	}
	Point(int _family, long long _coord):
		family(_family), coord(_coord) {}
};

long long Point::dist2;

void ensureThereIsNoKisses(long long dist2, vector<long long> &L1, vector<long long> &L2) {
	Point::dist2 = dist2;
	vector<Point> P1, P2;
	P1.reserve(2*L1.size());
	P2.reserve(2*L2.size());
	for (int i = L1.size() - 1; i >= 0; --i) {
		P1.push_back(Point(0, -L1[i]));
	}
	for (int i = 0; i < L1.size(); ++i) {
		P1.push_back(Point(0, +L1[i]));
	}
	for (int i = L2.size() - 1; i >= 0; --i) {
		P2.push_back(Point(1, -L2[i]));
	}
	for (int i = 0; i < L2.size(); ++i) {
		P2.push_back(Point(1, +L2[i]));
	}
	int i = 0, j = 0;
	while ((i < P1.size()) && (j < P2.size())) {
		ensuref(
			P1[i] != P2[j],
			"Circles with radiuses %d and %d touches",
			abs(P1[i].coord), abs(P2[j].coord)
		);
		if (P1[i] < P2[j]) {
			++i;
		} else {
			++j;
		}
	}
}

int main(int argc, char* argv[]) {
	setName("cococoncentric problem's tests validator");
	registerValidation(argc, argv);

	/* ====================== */
	/* ===== SET LIMITS ===== */
	/* ====================== */
	/*if (validator.group() == "group1") {
		min_coords = -1000;
		max_coords = +1000;
		max_nm = 1000;
		deny_overlaps = true;
		deny_kisses = true;
		max_radius = 10000;
	} else if (validator.group() == "group2") {
		min_coords = -1000;
		max_coords = +1000;
		max_nm = 1000;
		deny_overlaps = false;
		deny_kisses = true;
		max_radius = 10000;
	} else if (validator.group() == "group3") {
		min_coords = -1000 * 1000;
		max_coords = +1000 * 1000;
		max_nm = 100 * 1000;
		deny_overlaps = true;
		deny_kisses = true;
		max_radius = 300 * 1000;
	} else if (validator.group() == "group4") {
		min_coords = -1000 * 1000;
		max_coords = +1000 * 1000;
		max_nm = 100 * 1000;
		deny_overlaps = false;
		deny_kisses = true;
		max_radius = 300 * 1000;
	} else if (validator.group() == "group5") {
		min_coords = -1000 * 1000;
		max_coords = +1000 * 1000;
		max_nm = 1000;
		deny_overlaps = false;
		deny_kisses = false;
		max_radius = 300 * 1000;
	} else if (validator.group() == "group6") {*/
		min_coords = -1000*1000*1000;
		max_coords = +1000*1000*1000;
		max_nm = 100 * 1000;
		deny_overlaps = false;
		deny_kisses = false;
		max_radius = 2*1000*1000*1000;
	/*} else {
		quit(_fail, format("Wrong group name: %s", validator.group()));
	}*/

	/* ====================== */
	/* ======== INPUT ======= */
	/* ====================== */
	long long x1 = inf.readInt(min_coords, max_coords, "x1");
	inf.readSpace();
	long long y1 = inf.readInt(min_coords, max_coords, "y1");
	inf.readEoln();
	vector<long long> L1;
	int n;
	readRadiuses(n, L1, "L1");

	long long x2 = inf.readInt(min_coords, max_coords, "x2");
	inf.readSpace();
	long long y2 = inf.readInt(min_coords, max_coords, "y2");
	inf.readEoln();
	vector<long long> L2;
	int m;
	readRadiuses(m, L2, "L2");

	inf.readEof();

	long long dist2 = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2);
	/* ====================== */
	/* === CHECK OVERLAPS === */
	/* ====================== */
	if (deny_overlaps) {
		for (int i = 0; i < L1.size(); ++i) {
			ensuref(
				L1[i]*L1[i] < dist2,
				"%dst circle [radius=%d] with center in first point contains second point",
				i, L1[i]
			);
		}
		for (int i = 0; i < L2.size(); ++i) {
			ensuref(
				L2[i]*L2[i] < dist2,
				"%dst circle [radius=%d] with center in second point contains first point",
				i, L2[i]
			);
		}
	}

	/* ====================== */
	/* ==== CHECK TOUCHES === */
	/* ====================== */

	if (deny_kisses) ensureThereIsNoKisses(dist2, L1, L2);

	return 0;
}
