#!/usr/bin/env python3
import sys
from functools import lru_cache
from sys import setrecursionlimit
setrecursionlimit(10**9)


class BaseSolution:
    def __init__(self, in_stream=sys.stdin, out_stream=sys.stdout):
        self.in_stream = in_stream
        self.out_stream = out_stream

    def read(self):
        raise NotImplemented()

    def solve(self):
        raise NotImplemented()

    def write(self):
        raise NotImplemented()

    def readInts(self):
        return list(map(int, self.input().split()))

    def print(self, *args, **kwargs):
        print(*args, file=self.out_stream, **kwargs)

    def input(self, *args, **kwargs):
        return self.in_stream.readline().rstrip('\n\r')

class BaseCoCoConcentricSolution(BaseSolution):
    def read(self):
        self.point1 = self.readInts()
        self.L1 = self.readInts()[1:]
        self.point2 = self.readInts()
        self.L2 = self.readInts()[1:]
        self.N = len(self.L1) + len(self.L2)

    def solve(self):
        self.dist = self.distanceCalculate()
        self.strict_intersections = self.strictIntersectionsCount()
        self.vertexes = 2 * self.N + 2 * self.strict_intersections
        self.connected_components = self.connectedComponentsCount()
        self.edges = 4*self.strict_intersections+ (2*self.N)
        self.faces = self.connected_components + self.edges - self.vertexes

    def distanceCalculate(self):
        from math import sqrt
        return sqrt((self.point1[0]-self.point2[0])**2 + (self.point1[1]-self.point2[1])**2)

    def write(self):
        self.print(self.faces)

        print('Vertexes:', self.vertexes, file=sys.stderr)
        print('Edges:', self.edges, file=sys.stderr)
        print('Components:', self.connected_components, file=sys.stderr)
        print('Strict Intersections:', self.strict_intersections, file=sys.stderr)

    def strictIntersectionsCount(self):
        raise NotImplemented()

    def connectedComponentsCount(self):
        raise NotImplemented()


class SquareCoCoConcentricSolution(BaseCoCoConcentricSolution):
    EPS = 1e-7

    def isStrictIntersects(self, i, j):
        return  (self.L1[i] + self.L2[j] > self.dist + self.EPS) \
                and (abs(self.L1[i] - self.L2[j]) < self.dist - self.EPS)

    def isTouches(self, i, j):
        return  abs(self.L1[i] + self.L2[j] - self.dist) < self.EPS \
                or abs(abs(self.L1[i] - self.L2[j]) - self.dist) < self.EPS

    def isIntersects(self, i, j):
        return self.isStrictIntersects(i, j) or self.isTouches(i, j)

    def predicatCount(self, predicat):
        cnt = 0
        for i in range(len(self.L1)):
            for j in range(len(self.L2)):
                if predicat(i, j):
                    cnt += 1
        return cnt

    def strictIntersectionsCount(self):
        return self.predicatCount(self.isStrictIntersects)

    def connectedComponentsCount(self):
        colors1 = [None] * len(self.L1)
        colors2 = [None] * len(self.L2)
        color = 0

        def dfs1(v):
            colors1[v] = color
            for u in range(len(colors2)):
                if colors2[u] is None and self.isIntersects(v, u):
                    dfs2(u)

        def dfs2(v):
            colors2[v] = color
            for u in range(len(colors1)):
                if colors1[u] is None and self.isIntersects(u, v):
                    dfs1(u)

        for colors, dfs in ((colors1, dfs1), (colors2, dfs2)):
            for i in range(len(colors)):
                if colors[i] is None:
                    dfs(i)
                    color += 1

        return color

class CoCoConcentricSolution(SquareCoCoConcentricSolution):
    pass

if __name__ == '__main__':
    solution = CoCoConcentricSolution()
    solution.read()
    solution.solve()
    solution.write()

