#include <iostream>
#include <algorithm>
#include <string>
#include <vector>

using namespace std;

#define fi first
#define se second
#define mp make_pair
#define pb push_back

const int INF = 1000 * 1000 * 1000 + 5;

void build_suffix_array(const string &s, vector<int> &ans) {
    int n = (int)s.size();
    vector<int> c(n); // colors
    vector<int> d(n); // colors copy
    vector<int> e(n); // sorting pointers
    vector<int> f(n); // sorted
    vector<int> g(n); // sorted copy

    vector< pair<char, int> > h(n); // initial sorted
    for (int i = 0; i < n; ++i) {
        h[i] = {s[i], i};
    }
    sort(h.begin(), h.end());
    int cnt = 0;
    for (int i = 0; i < n; ++i) {
        if (i > 0 && h[i - 1].first != h[i].first) {
            e[cnt++] = i;
        }
        f[i] = h[i].second;
        c[f[i]] = cnt;
    }
    e[cnt] = n;
    
    for (int x = 1; x < n; x *= 2) {
        for (int i = n - 1; i >= 0; --i) {
            int j = (f[i] - x + n) % n;
            g[--e[c[j]]] = j;
        }
        f.swap(g);
        int cnt = 0;
        for (int i = 0; i < n; ++i) {
            if (i > 0 && (c[f[i - 1]] != c[f[i]] ||
                          c[(f[i - 1] + x) % n] != c[(f[i] + x) % n])) {
                e[cnt++] = i;
            }
            d[f[i]] = cnt;
        }
        e[cnt] = n;
        c.swap(d);
    }
    
    ans.swap(f);
}

// CAREFUL! Cyclic shifts should be different!
void build_lcp(const string &s, const vector<int> &a, vector<int> &ans) {
    int n = (int)s.size();
    ans.resize(n);
    
    vector<int> b(n); // inversed suffix array
    for (int i = 0; i < n; ++i) {
        b[a[i]] = i;
    }
    
    for (int i = 0; i < n; ++i) {
        int j = b[i];
        if (j == n - 1) {
            ans[j] = 0;
            continue;
        }
        int k = a[j + 1];
        ans[j] = (i > 0 ? max(0, ans[b[i - 1]] - 1) : 0);
        while (s[i + ans[j]] == s[k + ans[j]]) {
            ++ans[j];
        }
    }
}

struct Node {
    int lcp, le, ri;
    
    Node(int lcp = INF, int le = INF, int ri = -INF)
        : lcp(lcp), le(le), ri(ri) {}
};

const int N = 100000;
const int SUM_LEN = 1000000;
const int S_LEN = SUM_LEN + N + 1;
const int T_LEN = 1 << 22;
const char CH = '$';
int n; // number of string
string s; // s_1$s_2$...$s_n$$
int m; // s length
vector<int> a; // suffix array
vector<int> b; // inversed suffix array
vector<int> lcp;
Node t[T_LEN]; // segment tree
vector<int> r; // string pointers
vector<int> c; // string by position

void build_seg_tree(int v, int lt, int rt) {
    if (lt + 1 == rt) {
        t[v] = Node(lcp[lt], a[lt], a[lt]);
    } else {
        int mt = (lt + rt) / 2;
        build_seg_tree(2 * v, lt, mt);
        build_seg_tree(2 * v + 1, mt, rt);
        t[v] = Node(min(t[2 * v].lcp, t[2 * v + 1].lcp),
                    min(t[2 * v].le, t[2 * v + 1].le),
                    max(t[2 * v].ri, t[2 * v + 1].ri));
    }
}

int get_lcp(int v, int lt, int rt, int l, int r) {
    if (l >= r) {
        return INF;
    }
    if (l == lt && r == rt) {
        return t[v].lcp;
    }
    int mt = (lt + rt) / 2;
    return min(get_lcp(2 * v, lt, mt, l, min(mt, r)),
               get_lcp(2 * v + 1, mt, rt, max(mt, l), r));
}

int get_le(int v, int lt, int rt, int l, int r) {
    if (l >= r) {
        return INF;
    }
    if (l == lt && r == rt) {
        return t[v].le;
    }
    int mt = (lt + rt) / 2;
    return min(get_le(2 * v, lt, mt, l, min(mt, r)),
               get_le(2 * v + 1, mt, rt, max(mt, l), r));
}

int get_ri(int v, int lt, int rt, int l, int r) {
    if (l >= r) {
        return -INF;
    }
    if (l == lt && r == rt) {
        return t[v].ri;
    }
    int mt = (lt + rt) / 2;
    return max(get_ri(2 * v, lt, mt, l, min(mt, r)),
               get_ri(2 * v + 1, mt, rt, max(mt, l), r));
}

int main() {
#ifdef HOME
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(0);

    cin >> n >> ws;
    r.resize(n + 1);
    for (int i = 0; i < n; ++i) {
        string t;
        getline(cin, t);
        s += t;
        s += CH;
        if (i == n - 1) {
            s += CH;
        }
        r[i + 1] = r[i] + (int)t.size() + 1;
    }
    m = (int)s.size();

    c.resize(m);
    for (int i = 0, j = 0; i < m; ++i) {
        if (i == r[j + 1]) {
            ++j;
        }
        c[i] = j;
    }
    
    build_suffix_array(s, a);
    b.resize(m);
    for (int i = 0; i < m; ++i) {
        b[a[i]] = i;
    }
    build_lcp(s, a, lcp);
    build_seg_tree(1, 0, m);
    for (int i = 0; i < n; ++i) {
        int k = r[i + 1] - r[i] - 1;
        int le = 0, ri = k + 1;
        int ri_p = INF;
        while (le + 1 < ri) {
            int x = (le + ri) / 2;
            int p_mi = INF;
            for (int j = 0; j + x <= k; ++j) {
                int p = b[r[i] + j];
                int le = -1, ri = p;
                while (le + 1 < ri) {
                    int mi = (le + ri) / 2;
                    int lcp = get_lcp(1, 0, m, mi, p);
                    if (lcp < x) {
                        le = mi;
                    } else {
                        ri = mi;
                    }
                }
                int u = ri;
                le = p, ri = m;
                while (le + 1 < ri) {
                    int mi = (le + ri) / 2;
                    int lcp = get_lcp(1, 0, m, p, mi);
                    if (lcp < x) {
                        ri = mi;
                    } else {
                        le = mi;
                    }
                }
                int v = le;
                le = get_le(1, 0, m, u, v + 1);
                ri = get_ri(1, 0, m, u, v + 1);
                if (c[le] == c[ri]) {
                    p_mi = min(p_mi, p);
                }
            }
            if (p_mi != INF) {
                ri = x;
                ri_p = p_mi;
            } else {
                le = x;
            }
        }
        if (ri == k + 1) {
            cout << '?' << '\n';
        } else {
            cout << s.substr(a[ri_p], ri) << '\n';
        }
    }
    return 0;
}
