/*
  Time: O(L)
  Memory: O(sigma * L)
*/

#include <iostream>
#include <vector>
#include <string>
#include <cstdio>

using namespace std;

typedef pair<int, int> pii;

const int SIG = 27;
const char CH = 'a' - 1;

struct Node;

struct Edge {
    Node *v;
    int l, k;

    Edge(Node *v = NULL, int l = -1, int k = -1)
        : v(v), l(l), k(k) {}
};

struct Node {
    Edge g[SIG];
    Node *ln;
    int l, r;
    int id;

    Node(Node *ln = NULL, int p = -1)
        : ln(ln), l(p), r(p), id(-1) {
    }
};

const int N = 100000;
const int STR_LEN = 500000;
const int SUM_LEN = 500000;
const int TR_LEN = SUM_LEN + N + 1;
int n;
int m;
int a[N];
int b[TR_LEN];
int d[TR_LEN + 1];
int f[N];
Node* g[N];
string ans[N];

struct SuffixTree {
    Node *root;
    string t;

    SuffixTree() : root(new Node()) {
        root->ln = root;
    }

    void build() {
        int n = (int)t.size();
        for (int q = 0, r = 0; q < n; ++q) {
            int c = t[q] - CH;
            Node *v = root;
            int i = r;
            Node *pr_pa = NULL;
            while (r <= q) {
                int w = t[i] - CH;
                while (q - i > 0 && q - i >= v->g[w].k) {
                    i += v->g[w].k;
                    v = v->g[w].v;
                    w = t[i] - CH;
                }
                Node *pa = v;
                if (q - i != 0) {
                    int z = t[v->g[w].l + (q - i)] - CH;
                    if (c != z) {
                        pa = new Node();
                        pa->g[z] = Edge(v->g[w].v,
                                        v->g[w].l + (q - i),
                                        v->g[w].k - (q - i));
                        v->g[w].v = pa;
                        v->g[w].k = q - i;
                    } else {
                        break;
                    }
                }
                if (pr_pa != NULL) {
                    pr_pa->ln = pa;
                }
                pr_pa = pa;
                if (pa->g[c].v == NULL) {
                    pa->g[c] = Edge(new Node(root, r), q, n - q);
                } else {
                    break;
                }
                if (v == v->ln) {
                    ++i;
                }
                v = v->ln;
                ++r;
            }
        }
    }

    void dfs(Node *v, bool good, int h) {
        if (v->l != -1) {
            return;
        }
        v->l = (int)t.size();
        v->r = -1;
        for (int c = 0; c < SIG; ++c) {
            Node *u = v->g[c].v;
            if (u != NULL) {
                bool cl = (d[v->g[c].l + v->g[c].k] - d[v->g[c].l] == 0);
                dfs(u, good && cl, h + v->g[c].k);
                v->l = min(v->l, u->l);
                v->r = max(v->r, u->r);
                if (good && c != 0 && b[u->l] == b[u->r]) {
                    int id = b[u->l];
                    if (f[id] == 0 || h + 1 < f[id]) {
                        f[id] = h + 1;
                        g[id] = u;
                    }
                }
            }
        }
    }

    void restore(Node *v, vector<pii> &tr) {
        for (int c = 0; c < SIG; ++c) {
            Node *u = v->g[c].v;
            if (u != NULL) {
                if (u->id != -1) {
                    for (const pii &x : tr) {
                        ans[u->id] += t.substr(x.first, x.second);
                    }
                    ans[u->id] += (char)(c + CH);
                }
                
                tr.push_back({v->g[c].l, v->g[c].k});
                restore(u, tr);
                tr.pop_back();
            }
        }
    }
};

SuffixTree st;

int main() {
#ifdef HOME
    freopen("input.txt", "r", stdin);
    //freopen("output.txt", "w", stdout);
#else
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    
    cin >> n >> ws;
    for (int i = 0; i < n; ++i) {
        string s;
        getline(cin, s);
        st.t += s;
        st.t += CH;
        if (i == n - 1) {
            st.t += CH;
        }
        a[i] = (int)s.size() + 1;
    }
    m = (int)st.t.size();
    st.build();
    for (int i = 0, j = 0, k = 0; i < m; ++i, ++k) {
        if (k == a[j]) {
            ++j;
            k = 0;
        }
        b[i] = j;
        d[i + 1] = d[i] + (st.t[i] == CH);
    }
    st.dfs(st.root, true, 0);
    for (int i = 0; i < n; ++i) {
        if (g[i] != NULL) {
            g[i]->id = i;
        }
    }
    vector<pii> tr;
    st.restore(st.root, tr);
    
    for (int i = 0; i < n; ++i) {
        if (ans[i].empty()) {
            cout << '?' << '\n';
        } else {
            cout << ans[i] << '\n';
        }
    }
    return 0;
}
