#include "testlib.h"
#include <vector>
#include <cstdlib>
#include <algorithm>
#include <cmath>

using namespace std;

pair<int, int> sorted(pair<int, int> x) {
    if (x.first > x.second) {
        swap(x.first, x.second);
    }
    return x;
}

const int LOG = 0;
const int SQRT = 1;
int mode1;
int mode2;
int mode3;

int upperBound(int n, int mode) {
    if (mode == LOG) {
        return (int) 2 * log(n);   
    } else if (mode == SQRT) {
        return (int) 2 * sqrt(n);
    }
    return 2 * n / mode;
}


int main(int argc, char **argv) {                                                                                                                                                                                                
    registerGen(argc, argv, 1);
    int minN = atoi(argv[1]);
    int maxN = atoi(argv[2]);
    int maxM = atoi(argv[3]);
    int minK = atoi(argv[4]);
    int maxK = atoi(argv[5]);
    mode1 = atoi(argv[6]);
    mode2 = atoi(argv[7]);
    mode3 = atoi(argv[8]);
    int k = rnd.next(minK, maxK);
    
    int n;
    set<pair<pair<int, int>, int> > edges;
    int dist;
    do {
        edges.clear();
        n = rnd.next(minN, maxN);
        vector <vector <int> > layers;
        layers.push_back(vector <int> (1, 0));
        int used = 1;
        while ((n - 1) - used != 0) {
            int nextChunk = rnd.next(1, min(max(1, upperBound(n, mode1)), (n - 1) - used));
            used += nextChunk;
            layers.push_back(vector<int> ());
            for (int i = 0; i < nextChunk; ++i) {
                layers.back().push_back(layers[layers.size() - 2].back() + i + 1);
            }
        }
        layers.push_back(vector<int>(1, n - 1));
        dist = layers.size() - 1;
        vector <vector <int> > layersChunks;
        layersChunks.push_back(vector <int> (1, 0));
        used = 1;
        while (dist - used != 0) {
            int nextChunk = rnd.next(1, min(max(1, upperBound(dist + 1, mode2)), dist - used));
            used += nextChunk;
            layersChunks.push_back(vector <int> ());
            for (int i = 0; i < nextChunk; ++i) {
                layersChunks.back().push_back(layersChunks[layersChunks.size() - 2].back() + i + 1);
            }
        }
        layersChunks.push_back(vector<int>(1, dist));
        
        vector <vector <vector <int> > > distribution(dist + 1);
        
        for (int i = 0; i < layersChunks.size(); ++i) {
            int minSize = INT_MAX;
            for (int j = 0; j < layersChunks[i].size(); ++j) {
                minSize = min(minSize, (int)layers[layersChunks[i][j]].size());
            }
            int avgSize = rnd.next(1, max(1, upperBound(minSize, mode2)));
            int chunks = min(minSize, max(1, minSize / avgSize));
            for (int j = 0; j < layersChunks[i].size(); ++j) {
                int layer = layersChunks[i][j];
                int leftToPick = layers[layer].size();
                for (int h = 0; h < chunks; ++h) {
                    int currentSize = min(leftToPick - (chunks - h - 1), max(1, upperBound(layers[layer].size(), mode3)));
                    if (h + 1 == chunks)
                        currentSize = leftToPick;
                    
                    distribution[layer].push_back(vector <int> ());
                    for (int x = 0; x < currentSize; ++x) {
                        distribution[layer].back().push_back(layers[layer][layers[layer].size() - leftToPick + x]);
                    }
                    leftToPick -= currentSize;
                }
                shuffle(distribution[layer].begin(), distribution[layer].end());
            }
        }
        
        vector <vector <int> > responsibleColors(layersChunks.size());
        
        vector <int> candidates1;
        vector <int> candidates2;
        for (int i = 0; i < layersChunks.size(); ++i) {
            for (int j = 0; j < layersChunks[i].size() - 1; ++j) {
                if (distribution[layersChunks[i][0]].size() != 1)
                    candidates1.push_back(i);
                else 
                    candidates2.push_back(i);
            }
            if (i + 1 != layersChunks.size()) {
                candidates2.push_back(i);
            }
        }
        shuffle(candidates1.begin(), candidates1.end());
        candidates1.resize(min((int)candidates1.size(), k));
        for (int i = 0; i < candidates1.size(); ++i) {
            responsibleColors[candidates1[i]].push_back(i);
        }
        shuffle(candidates2.begin(), candidates2.end());
        bool contFlag = false;
        for (int i = candidates1.size(); i < k; ++i) {
            if (i - candidates1.size() == candidates2.size())
                contFlag = true;
            responsibleColors[candidates2[i - candidates1.size()]].push_back(i);
        }
        
        if (contFlag)
            continue;
        
        vector <vector <int> > chunkColor(dist);
        for (int i = 0; i < responsibleColors.size(); ++i) {
            for (int j = 0; j < layersChunks[i].size(); ++j) {
                if (layersChunks[i][j] != dist)
                    chunkColor[layersChunks[i][j]].assign(distribution[layersChunks[i][j]].size(), -1);
            }
            for (int j = 0; j < distribution[layersChunks[i][0]].size(); ++j) {
                vector <int> availible;
                for (int h = 0; h + 1 < layersChunks[i].size(); ++h) {
                    availible.push_back(layersChunks[i][h]);
                }
                shuffle(availible.begin(), availible.end());
                availible.resize(min(availible.size(), responsibleColors[i].size()));
                for (int h = 0; h < availible.size(); ++h) {
                    chunkColor[availible[h]][j] = responsibleColors[i][h] + 1;
                }
                if (availible.size() != responsibleColors[i].size()) {
                    chunkColor[layersChunks[i][layersChunks[i].size() - 1]][j] = responsibleColors[i].back() + 1;
                }
            }
        }
        
        
        vector <pair<int, int> > coordinates(n);
        for (int i = 0; i < distribution.size(); ++i) {
            for (int j = 0; j < distribution[i].size(); ++j) {
                for (int h = 0; h < distribution[i][j].size(); ++h) {
                    coordinates[distribution[i][j][h]] = make_pair(i, j);
                }
            }
        }
        
        vector <bool> isLast(layers.size(), false);
        
        for (int i = 0; i < layersChunks.size(); ++i) {
            isLast[layersChunks[i].back()] = true;
        }
        
//         for (int i = 0; i < layers.size(); ++i)
//         {
//             for (int j = 0; j < layers[i].size(); ++j) {
//                 printf("%d ", layers[i][j]);
//             }
//             printf("\n");
//         }
//         printf("----------\n");
//         for (int z = 0; z < layersChunks.size(); ++z) {
//             for (int x = 0; x < layersChunks[z].size(); ++x) {
//                 int i = layersChunks[z][x];
//                 for (int j = 0; j < distribution[i].size(); ++j) {
//                     for (int h = 0; h < distribution[i][j].size(); ++h) {
//                         printf("%d ", distribution[i][j][h]);
//                     }
//                     printf("| ");
//                 }
//                 printf("\n");
//             }
//             printf("---------\n");
//         }
//         for (int i = 0; i < distribution.size(); ++i) {
//             for (int j = 0; j < distribution[i].size(); ++j) {
//                 for (int h = 0; h < distribution[i][j].size(); ++h) {
//                     printf("%d ", distribution[i][j][h]);
//                 }
//                 printf("| ");
//             }
//             printf("\n");
//         }
//         
//         for (int i = 0; i + 1 < distribution.size(); ++i) {
//             for (int j = 0; j < distribution[i].size(); ++j) {
//                 printf("%d ", chunkColor[i][j]);
//                 printf("| ");
//             }
//             printf("\n");
//         }
        
        long long possible = 0;
        
        for (int i = 1; i < layersChunks.size(); ++i) {
            for (int j = 0; j < layersChunks[i].size(); ++j) {
                int layer = layersChunks[i][j];
                for (int h = 0; h < distribution[layer].size(); ++h) {
                    for (int x = 0; x < distribution[layer][h].size(); ++x) {
                        int parent = (j == 0 ? layers[layer - 1][rnd.next(0, (int)layers[layer - 1].size() - 1)] : distribution[layer - 1][h][rnd.next(0, (int)distribution[layer - 1][h].size() - 1)]);
                        int ancestor = distribution[layer][h][x];
                        //printf("%d %d %d %d %d\n", layer - 1, h, chunkColor[layer - 1][h], chunkColor[layer - 1][0], j);
                        int sample = (j != 0 ? chunkColor[layer - 1][h] : chunkColor[layer - 1][0]);
                        int color = (sample == -1 ? 0 : sample);
                        edges.insert(make_pair(sorted(make_pair(ancestor, parent)), color));
                    }
                    possible += 1ll * distribution[layer][h].size() * (j == 0 ? layers[layer - 1].size() : distribution[layer - 1][h].size());
                    possible += 1ll * distribution[layer][h].size() * (distribution[layer][h].size() - 1) / 2;
                }
            }
        }
        
        for (int i = 0; i + 1 < layersChunks.size(); ++i) {
            for (int j = 0; j < layersChunks[i].size(); ++j) {
                int layer = layersChunks[i][j];
                for (int h = 0; h < distribution[layer].size(); ++h) {
                    for (int x = 0; x < distribution[layer][h].size(); ++x) {
                        int parent = (j + 1 == layersChunks[i].size() ? layers[layer + 1][rnd.next(0, (int)layers[layer + 1].size() - 1)] : distribution[layer + 1][h][rnd.next(0, (int)distribution[layer + 1][h].size() - 1)]);
                        int ancestor = distribution[layer][h][x];
                        int color = (chunkColor[layer][h] == -1 ? 0 : chunkColor[layer][h]);
                        set<pair<pair<int, int>, int> >::iterator it = edges.lower_bound(make_pair(sorted(make_pair(ancestor, parent)), -1));
                        if (it == edges.end() || it->first != sorted(make_pair(ancestor, parent)))
                            edges.insert(make_pair(sorted(make_pair(ancestor, parent)), color));
                    }
                }
            }
        }
        
        while (edges.size() < maxM && edges.size() < possible) {
            int i = rnd.next(1, n - 2);
            int j;
            int color;
            int layer = coordinates[i].first;
            int chunk = coordinates[i].second;
            
            if (rnd.next(0, 1) == 0) {
                j = distribution[layer][chunk][rnd.next(0, (int)distribution[layer][chunk].size() - 1)];
                color = 0;
            } else {
                if (isLast[layer]) {
                    j = layers[layer + 1][rnd.next(0, (int)layers[layer + 1].size() - 1)]; 
                } else {
                    j = distribution[layer + 1][chunk][rnd.next(0, (int)distribution[layer + 1][chunk].size() - 1)];
                }
                color = chunkColor[layer][chunk] == -1 ? 0 : chunkColor[layer][chunk];
            }
            
            if (i != j) {
                set<pair<pair<int, int>, int> >::iterator it = edges.lower_bound(make_pair(sorted(make_pair(i, j)), -1));
                if (it == edges.end() || it->first != sorted(make_pair(i, j)))
                    edges.insert(make_pair(sorted(make_pair(i, j)), color));
            }
        }
        
        
//         printf("%d\n", edges.size());
//         for (auto const &x: edges) {
//             printf("%d -- %d [label = \"%d\"]\n", x.first.first, x.first.second, x.second);
//         }
    } while (edges.size() > maxM || k > dist);
    
    vector <pair<pair<int, int>, int> > edg;
    for (set<pair<pair<int, int>, int> >::iterator it = edges.begin(); it != edges.end(); ++it) {
        pair<pair<int, int>, int> current = *it;
        if (rnd.next(1, 2) == 1) {
            swap(current.first.first, current.first.second);
        }
        edg.push_back(current);
    }
    vector <int> vp(n);
    for (int i = 0; i < n; ++i)
        vp[i] = i;
    shuffle(vp.begin(), vp.end());
    vector <int> cp(k + 1);
    for (int i = 1; i <= k; ++i)
        cp[i] = i;
    shuffle(cp.begin() + 1, cp.end());
    shuffle(edg.begin(), edg.end());
    printf("%d %d %d\n", n, (int)edg.size(), k);
    for (vector <pair<pair<int, int>, int> >::iterator it = edg.begin(); it != edg.end(); ++it) {
        printf("%d %d %d\n", vp[it->first.first] + 1, vp[it->first.second] + 1 , (it->second == 0 ? it->second : cp[it->second]));
    }
    
    printf("%d %d\n", vp[0] + 1, vp[n - 1] + 1);
    return 0;
}