#include "testlib.h"
#include <vector>
#include <cstdlib>
#include <algorithm>
#include <cmath>

using namespace std;

pair<int, int> sorted(pair<int, int> x) {
    if (x.first > x.second) {
        swap(x.first, x.second);
    }
    return x;
}

const int LOG = 0;
const int SQRT = 1;
int mode; 
int k;

int upperBound(int n) {
    if (mode == LOG) {
        return (int) 2 * log(n);   
    } else if (mode == SQRT) {
        return (int) 2 * sqrt(n);
    }
    return 2 * n / mode;
}


vector <vector <pair<int, int> > > g;
vector <int> dist_nce;
int n;

void dfs(int v, int c, int parent, int nc) {
    vector <int> possibleNext;
    for (int i = 0; i < g[v].size(); ++i) {
        if (dist_nce[g[v][i].first] == dist_nce[v] + 1) {
            possibleNext.push_back(i);
        }
        if (g[v][i].first == parent) {
            g[v][i].second = nc;
        }
    }
    if (v == n - 1) 
        return;
    int j = possibleNext[rnd.next(0, (int)possibleNext.size() - 1)];
    while (g[v][j].second == c) {
        g[v][j].second = rnd.next(0, k);
    }
    dfs(g[v][j].first, c, v, g[v][j].second);
}


int main(int argc, char **argv) {                                                                                                                                                                                                
    registerGen(argc, argv, 1);
    int minN = atoi(argv[1]);
    int maxN = atoi(argv[2]);
    int maxM = atoi(argv[3]);
    int minK = atoi(argv[4]);
    int maxK = atoi(argv[5]);
    mode = atoi(argv[6]);
    int bad = atoi(argv[7]);
    int rev = true;
    k = rnd.next(minK, maxK);

    set<pair<pair<int, int>, int> > edges;
    int dist;
    do {
        edges.clear();
        
        n = rnd.next(minN, maxN);
        dist_nce.resize(n);
        vector <vector <int> > layers;
        layers.push_back(vector <int> (1, 0));
        dist_nce[0] = 0;
        int used = 1;
        while ((n - 1) - used != 0) {
            int nextChunk = rnd.next(1, min(max(1, upperBound(n)), (n - 1) - used));
            used += nextChunk;
            layers.push_back(vector<int> ());
            for (int i = 0; i < nextChunk; ++i) {
                dist_nce[layers[layers.size() - 2].back() + i + 1] = layers.size() - 1;
                layers.back().push_back(layers[layers.size() - 2].back() + i + 1);
            }
        }
        layers.push_back(vector<int>(1, n - 1));
        dist_nce[n - 1] = layers.size() - 1;
        dist = layers.size() - 1;
        long long possible = 0;
        for (int layer = 1; layer <= dist; ++layer) {
            for (int j = 0; j < layers[layer].size(); ++j) {
                int parent = layers[layer - 1][rnd.next(0, (int)layers[layer - 1].size() - 1)];
                int ancestor = layers[layer][j];
                edges.insert(make_pair(sorted(make_pair(ancestor, parent)), layer - 1));
            }
            possible = 1ll * layers[layer].size() * layers[layer - 1].size();
            possible += 1ll * layers[layer].size() * (layers[layer].size() - 1) / 2;
        }
        if (rev) {
            for (int layer = 0; layer < dist; ++layer) {
                for (int j = 0; j < layers[layer].size(); ++j) {
                    int parent = layers[layer + 1][rnd.next(0, (int)layers[layer + 1].size() - 1)];
                    int ancestor = layers[layer][j];
                    edges.insert(make_pair(sorted(make_pair(ancestor, parent)), layer));
                }
            }
        }
        while (edges.size() < min(1ll * maxM, possible)) {
            int i, j;
            int layer;
            if (rnd.next(0, 1) == 0) {
                layer = rnd.next(0, dist);
                i = layers[layer][rnd.next(0, (int)layers[layer].size() - 1)];
                j = layers[layer][rnd.next(0, (int)layers[layer].size() - 1)];
            } else {
                layer = rnd.next(0, dist - 1);
                i = layers[layer][rnd.next(0, (int)layers[layer].size() - 1)];
                j = layers[layer + 1][rnd.next(0, (int)layers[layer + 1].size() - 1)];                     
            }
            if (i != j) {
                edges.insert(make_pair(sorted(make_pair(i, j)), layer));
            }
        }
    } while (edges.size() > maxM || k > dist);
    vector <int> good(dist);
    for (int i = 0; i < dist; ++i) {
        good[i] = i;
    }
    shuffle(good.begin(), good.end());
    vector <int> reverse(dist + 1, -1);
    for (int i = 0; i < k; ++i) {
        reverse[good[i]] = i;
    }
    vector <pair<pair<int, int>, int> > edg;
    for (set<pair<pair<int, int>, int> >::iterator it = edges.begin(); it != edges.end(); ++it) {
        pair<pair<int, int>, int> current = *it;
        if (rnd.next(1, 2) == 1) {
            swap(current.first.first, current.first.second);
        }
        if (reverse[current.second] != -1) {
            current.second = reverse[current.second] + 1;
        } else {
            current.second = rnd.next(0, k);
        }
        edg.push_back(current);
    }
    
    g.resize(n);
    for (int i = 0; i < edg.size(); ++i) {
        g[edg[i].first.first].push_back(make_pair(edg[i].first.second, edg[i].second));
        g[edg[i].first.second].push_back(make_pair(edg[i].first.first, edg[i].second));
    }
    edg.clear();
    for (int i = 0; i < min(bad, k); ++i) {
        dfs(0, i + 1, -1, -1);
    }
    edges.clear();
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < g[i].size(); ++j) {
            edges.insert(make_pair(sorted(make_pair(i, g[i][j].first)), g[i][j].second));
        }
    }
    for (set<pair<pair<int, int>, int> >::iterator it = edges.begin(); it != edges.end(); ++it) {
        pair<pair<int, int>, int> current = *it;
        if (rnd.next(1, 2) == 1) {
            swap(current.first.first, current.first.second);
        }
        edg.push_back(current);
    }
    vector <int> vp(n);
    for (int i = 0; i < n; ++i)
        vp[i] = i;
    shuffle(vp.begin(), vp.end());
    vector <int> cp(k + 1);
    for (int i = 1; i <= k; ++i)
        cp[i] = i;
    shuffle(cp.begin() + 1, cp.end());
    shuffle(edg.begin(), edg.end());
    printf("%d %d %d\n", n, (int)edg.size(), k);
    for (vector <pair<pair<int, int>, int> >::iterator it = edg.begin(); it != edg.end(); ++it) {
        printf("%d %d %d\n", vp[it->first.first] + 1, vp[it->first.second] + 1 , (it->second == 0 ? it->second : cp[it->second]));
    }
    
    printf("%d %d\n", vp[0] + 1, vp[n - 1] + 1);
    return 0;
}