#define _CRT_SECURE_NO_WARNINGS
#include <cstdio>
#include <vector>
#include <algorithm>
#include <map>
#include <set>
#include <climits>

using namespace std;

struct DynamicConnectivity
{

    struct Dsu
    {
        int n;
        vector <int> p, r;

        Dsu(){}
        Dsu(int n) : n(n)
        {
            p.resize(n);
            for (int i = 0; i < n; ++i)
                p[i] = i;
            r.assign(n, 0);
        }
        
        int getParent(int v)
        {
            if (v == p[v])
                return v;
            return getParent(p[v]);
        }

        void uniteSets(int v, int u)
        {
            //v = getParent(v);
            //u = getParent(u);
            if (v != u)
            {
                if (r[v] < r[u])
                    swap(v, u);
                p[u] = v;
                if (r[v] == r[u])
                    ++r[v];
            }
        }

        pair<int, int> operator[](int i)
        {
            return make_pair(p[i], r[i]);
        }

    };
    
    vector <pair<int, pair<int, int> > > q;
    int n;
    Dsu components;

    DynamicConnectivity(pair<vector <pair<int, pair<int, int> > >, int> qn) : q(qn.first), n(qn.second) 
    {
        components = Dsu(n);
    }
    
    struct SegmentTree
    {
        vector <vector <pair<int, int> > > t;
        int size;

        SegmentTree(int m)
        {
            for (size = 1; size < m; size *= 2);
            t.resize(2 * size);
        }

        void add(int v, int L, int R, int l, int r, pair<int, int> what)
        {
            if (l > r)
                return;
            if (l == L && r == R)
                return void(t[v].push_back(what));
            int M = (L + R) / 2;
            add(2 * v, L, M, l, min(M, r), what);
            add(2 * v + 1, M + 1, R, max(l, M + 1), r, what);
        }
    };

    map<pair<int, int>, vector <int> > times;
    vector <pair<int, pair<int, int> > > queryTimes;

    void sort(pair<int, int> &a)
    {
        if (a.first > a.second)
            swap(a.first, a.second);
    }
    
    int nextTimeYouShouldWorry;
    int ptr;

    //vector <int> ttimes;
    vector <bool> queryAnswers;

    void dfs(SegmentTree &st, int v, int L, int R)
    {
        vector<pair<int, pair<int, int> > > changed;
        for (int j = 0; j < st.t[v].size(); ++j)
        {
            pair<int, int> i = st.t[v][j];
            i.first = components.getParent(i.first);
            i.second = components.getParent(i.second);
            changed.push_back(make_pair(i.first, components[i.first]));
            changed.push_back(make_pair(i.second, components[i.second]));
            components.uniteSets(i.first, i.second);
        }
        if (v < st.size)
        {
            if (L <= nextTimeYouShouldWorry && nextTimeYouShouldWorry <= (L + R) / 2)
                dfs(st, 2 * v, L, (L + R) / 2);
            if ((L + R) / 2 + 1 <= nextTimeYouShouldWorry && nextTimeYouShouldWorry <= R)
                dfs(st, 2 * v + 1, (L + R) / 2 + 1, R);
            
        }
        else
        {
            if ((v - st.size + 1) == nextTimeYouShouldWorry)
            {
                queryAnswers.push_back(components.getParent(queryTimes[ptr].second.first) == components.getParent(queryTimes[ptr].second.second));
                nextTimeYouShouldWorry = queryTimes[++ptr].first;
            }
        }
        for (int i = int(changed.size()) - 1; i >= 0; --i)
        {
            components.p[changed[i].first] = changed[i].second.first;
            components.r[changed[i].first] = changed[i].second.second;
        }
    }

    vector <bool> main()
    {
        int T = 0;
        for (int j = 0; j < q.size(); ++j)
        {
            pair<int, pair<int, int> > i = q[j];
            sort(i.second);
            if (i.first)
            {
                if (times.find(i.second) == times.end())
                    times[i.second] = vector <int> ();
                times[i.second].push_back(++T);
                //ttimes.push_back(T);
            }
            else
            {
                queryTimes.push_back(make_pair(++T, i.second));
            }
        }

        queryTimes.push_back(make_pair(INT_MAX, make_pair(0, 0)));
        ptr = 0;
        nextTimeYouShouldWorry = queryTimes[0].first;
        SegmentTree st(T);
        for (map<pair<int, int>, vector <int> > :: iterator i = times.begin(); i != times.end(); ++i)// auto const &i: times)
        {
            
            for (int j = 0; j < i->second.size(); j += 2)
            {
                st.add(1, 1, st.size, i->second[j], i->second[j + 1], i->first);
            }
        }
        dfs(st, 1, 1, st.size);
        return queryAnswers;
    }
};

vector <vector <pair<int, int> > > graph;
vector <bool> used;

int A, B;

bool dfs(int v, int c, int d) {
    used[v] = true;
    if (v == A) {
        printf("%d\n", d);
        printf("%d ", v + 1);
        return true;
    }
    for (int i = 0; i < graph[v].size(); ++i) {
        if (graph[v][i].second != c) {
            int u = graph[v][i].first;
            if (!used[u]) {
                if (dfs(u, c, d + 1)) {
                    printf("%d ", v + 1);
                    return true;
                }
            }
        }
    }
    return false;
}

pair<vector <pair<int, pair<int, int> > >, int> input() {
    int n, m, k;
    scanf("%d %d %d", &n, &m, &k);
    vector <vector <pair<int, int> > > colors(k);

    vector <pair<int, pair<int, int> > > queries(m);
    vector <pair<int, int> > e(m);
    used.assign(n, 0);
    graph.resize(n);
    for (int i = 0; i < m; ++i) {
        int a, b, c;
        scanf("%d %d %d", &a, &b, &c);
        --a, --b;
        e[i] = make_pair(a, b);
        queries[i] = make_pair(1, make_pair(a, b));
        graph[a].push_back(make_pair(b, c));
        graph[b].push_back(make_pair(a, c));
        if (c) {
            colors[c - 1].push_back(make_pair(a, b));
        }
    }
    scanf("%d %d", &A, &B);
    --A, --B;
    for (int i = 0; i < k; ++i) {
        for (int j = 0; j < colors[i].size(); ++j) {
            queries.push_back(make_pair(-1, colors[i][j]));
        }
        queries.push_back(make_pair(0, make_pair(A, B)));
        for (int j = 0; j < colors[i].size(); ++j) {
            queries.push_back(make_pair(1, colors[i][j]));
        }
    }
    for (int i = 0; i < e.size(); ++i) {
        queries.push_back(make_pair(-1, e[i]));
    }
    return make_pair(queries, n);
}

int main()
{
    vector <bool> answer = DynamicConnectivity(input()).main();
    vector <int> ans;
    for (int i = 0; i < answer.size(); ++i) {
        if (answer[i]) {
            ans.push_back(i + 1);
//             printf("YES\n");
//             dfs(B, i + 1, 1);
//             return 0;
        }
    }
    sort(ans.begin(), ans.end());
    printf("%d\n", ans.size());
    for (int i = 0; i < ans.size(); ++i) {
        printf("%d", ans[i]);
        if (i + 1 == ans.size()) {
            printf("\n");
        } else {
            printf(" ");
        }
    }
    return 0;
}
