#include <algorithm>
#include <cassert>
#include <iostream>
#include <string>
#include <vector>
#include <cmath>
using namespace std;

#define mp make_pair

static const int kAlphabetSize = 300;

bool IsPalindrom(const string& s) {
  for (int i = 0; i < (int)s.size(); ++i) {
    if (s[i] != s[(int)s.size() - i - 1]) {
      return false;
    }
  }
  return true;
}

string Input() {
  string S;
  cin >> S;
  return S;
}

inline vector<int> LCS_last(const string& s, const string& t) {
  vector<int> prev(t.size() + 1, 0), next(t.size() + 1, 0);
  for (int i = 1; i <= (int)s.size(); ++i) {
    for (int j = 1; j <= (int)t.size(); ++j) {
      next[j] = max(prev[j], next[j - 1]);
      if (s[i - 1] == t[j - 1]) {
        next[j] = max(next[j], prev[j - 1] + 1);
      }
    }
    prev = next;
  }
  return prev;
}

inline int FindSplitPosition(const string& s, const string& t) {
  const vector<int> lcs_last = LCS_last(s, t);
  const int lcs_length = lcs_last[t.size()];

  const vector<int> lcs_left_last =
      LCS_last(s.substr(0, (int)s.size() / 2), t);
  
  string right_s = s.substr((int)s.size() / 2);
  reverse(right_s.begin(), right_s.end());

  string reversed_t = t;
  reverse(reversed_t.begin(), reversed_t.end());

  const vector<int> lcs_right_last = LCS_last(right_s, reversed_t);

  for (int i = 0; i <= t.size(); ++i) {
    if (lcs_left_last[i] + lcs_right_last[(int)t.size() - i] == lcs_length) {
      return i;
    }
  }
}

pair<vector<int>, vector<int> > LCS(const string& s, const string& t) {
  pair<vector<int>, vector<int> > positions;

  if ((int)s.size() <= 1 || (int)t.size() <= 1) {
    if ((int)s.size() == 1) {
      for (int i = 0; i < (int)t.size(); ++i) {
        if (s[0] == t[i]) {
          positions.first.push_back(0);
          positions.second.push_back(i);
          break;
        }
      }
    } else if ((int)t.size() == 1) {
      for (int i = 0; i < (int)s.size(); ++i) {
        if (s[i] == t[0]) {
          positions.first.push_back(i);
          positions.second.push_back(0);
          break;
        }
      }
    }
    return positions;
  } else {
    const int split_position = FindSplitPosition(s, t);

    positions = LCS(s.substr(0, (int)s.size() / 2), t.substr(0, split_position));

    pair<vector<int>, vector<int> > right_positions =
        LCS(s.substr((int)s.size() / 2), t.substr(split_position));

    for (int i = 0; i < (int)right_positions.first.size(); ++i) {
      right_positions.first[i] += (int)s.size() / 2;
    }
    for (int i = 0; i < (int)right_positions.second.size(); ++i) {
      right_positions.second[i] += split_position;
    }

    positions.first.insert(
        positions.first.end(),
        right_positions.first.begin(),
        right_positions.first.end());

    positions.second.insert(
        positions.second.end(),
        right_positions.second.begin(),
        right_positions.second.end());

    return positions;
  }
}

void LCS(const string& s, const string& t,
         vector<int>* s_positions, vector<int>* t_positions) {
  pair<vector<int>, vector<int> > positions = LCS(s, t);
  *s_positions = positions.first;
  *t_positions = positions.second;
}

string MergeStrings(const string& s, const string& t) {
  vector<int> s_positions;
  vector<int> t_positions;
  LCS(s, t, &s_positions, &t_positions);
  int s_ptr = 0;
  int t_ptr = 0;
  string result;
  for (int i = 0; i < (int)s_positions.size(); ++i) {
    while (s_ptr < s_positions[i]) {
      result += s[s_ptr];
      ++s_ptr;
    }
    while (t_ptr < t_positions[i]) {
      result += t[t_ptr];
      ++t_ptr;
    }
    result += s[s_ptr];
    ++s_ptr;
    ++t_ptr;
  }
  while (s_ptr < s.size()) {
    result += s[s_ptr];
    ++s_ptr;
  }
  while (t_ptr < t.size()) {
    result += t[t_ptr];
    ++t_ptr;
  }
  return result;
}

string Solve(const string& S) {
  const int length_of_S = S.size();
  vector<int> last_occurence(kAlphabetSize, -1);
  vector<int> next_occurence(length_of_S, -1);
  for (int i = 0; i < (int)length_of_S; ++i) {
    if (last_occurence[S[i]] != -1) {
      next_occurence[last_occurence[S[i]]] = i;
    }
    last_occurence[S[i]] = i;
  }
  // Case 1: odd number of symbols in palindrom.
  vector<int> v(length_of_S, 0);
  vector<int> next(length_of_S);
  int min_length_odd = 2 * length_of_S + 1;
  int min_center_position_odd = -1;
  for (int center = 0; center < length_of_S; ++center) {
    int cur_length;
    if (center + 1 == length_of_S) {
      cur_length = 2 * length_of_S - 1;
    } else {
      cur_length = 1 +
          2 * (center + (length_of_S - center - 1 - v[center + 1]));
    }
    if (cur_length < min_length_odd) {
      min_length_odd = cur_length;
      min_center_position_odd = center;
    }

    next.assign(length_of_S, 0);

    int next_center_pos = next_occurence[center];
    for (int i = center + 1; i < length_of_S; ++i) {
      while (next_center_pos != -1 && next_center_pos < i) {
        next_center_pos = next_occurence[next_center_pos];
      }
      if (next_center_pos == -1) {
        next[i] = v[i];
      } else {
        if (next_center_pos + 1 < length_of_S) {
          next[i] = max(1 + v[next_center_pos + 1], v[i]);
        } else {
          next[i] = max(1, v[i]);
        }
      }
    }

    v.swap(next);
  }

  // Case 2: even number of symbols in palindrom.
  v.assign(length_of_S, 0);
  int last_zero_pos = last_occurence[S[0]];
  for (int i = last_zero_pos; i > 0; --i) {
    v[i] = 1;
  }

  int min_length_even = 2 * length_of_S + 1;
  int min_center_position_even = -1;
  for (int center = 0; center < length_of_S; ++center) {
    int cur_length;
    if (center + 1 == length_of_S) {
      cur_length = 2 * length_of_S;
    } else {
      cur_length = 2 *
          (center + 1 + (length_of_S - center - 1 - v[center + 1]));
    }

    if (cur_length < min_length_even) {
      min_length_even = cur_length;
      min_center_position_even = center;
    }

    next.assign(length_of_S, 0);


    int next_center_pos;
    if (center + 1 < length_of_S) {
      next_center_pos = next_occurence[center + 1];
    }
    for (int i = center + 2; i < length_of_S; ++i) {
      while (next_center_pos != -1 && next_center_pos < i) {
        next_center_pos = next_occurence[next_center_pos];
      }
      if (next_center_pos == -1) {
        next[i] = v[i];
      } else {
        if (next_center_pos + 1 < length_of_S) {
          next[i] = max(1 + v[next_center_pos + 1], v[i]);
        } else {
          next[i] = max(1, v[i]);
        }
      }
    }

    v.swap(next);
  } 
  if (min_length_odd < min_length_even) {
    string left = S.substr(0, min_center_position_odd);
    reverse(left.begin(), left.end());
    string right = S.substr(min_center_position_odd + 1);
    string right_palindrom = MergeStrings(left, right);
    string left_palindrom = right_palindrom;
    reverse(left_palindrom.begin(), left_palindrom.end());
    string ans = left_palindrom + S[min_center_position_odd] + right_palindrom;
    assert(ans.size() == min_length_odd);
    return ans;
  } else {
    string left = S.substr(0, min_center_position_even + 1);
    reverse(left.begin(), left.end());
    string right = S.substr(min_center_position_even + 1);
    string right_palindrom = MergeStrings(left, right);
    string left_palindrom = right_palindrom;
    reverse(left_palindrom.begin(), left_palindrom.end());
    string ans = left_palindrom + right_palindrom;
    assert(ans.size() == min_length_even);
    return ans;
  }
}

void Output(const string& S, const string& palindrom) {
  cout << (int)palindrom.size() - (int)S.size() << endl;
  cout << palindrom << endl;
}

int main() {
  const string S = Input();
  const string palindrom = Solve(S);
  Output(S, palindrom);
}
