#include <algorithm>
#include <cassert>
#include <iostream>
#include <string>
#include <vector>
using namespace std;

#define mp make_pair

static const int kAlphabetSize = 300;

bool IsPalindrom(const string& s) {
  for (int i = 0; i < (int)s.size(); ++i) {
    if (s[i] != s[(int)s.size() - i - 1]) {
      return false;
    }
  }
  return true;
}

string Input() {
  string S;
  cin >> S;
  return S;
}

void LCS(const string& s, const string& t,
         vector<int>* s_positions, vector<int>* t_positions) {
  vector<vector<int> > dp(s.size() + 1, vector<int>(t.size() + 1, 0));
  vector<vector<pair<int, int> > > p(
      s.size() + 1, vector<pair<int, int> >(t.size() + 1, mp(-1, -1)));

  for (int i = 1; i <= (int)s.size(); ++i) {
    for (int j = 1; j <= (int)t.size(); ++j) {
      if (dp[i - 1][j] > dp[i][j - 1]) {
        dp[i][j] = dp[i - 1][j];
        p[i][j] = mp(i - 1, j);
      } else {
        dp[i][j] = dp[i][j - 1];
        p[i][j] = mp(i, j - 1);
      }

      if (s[i - 1] == t[j - 1] && dp[i][j] < dp[i - 1][j - 1] + 1) {
        dp[i][j] = dp[i - 1][j - 1] + 1;
        p[i][j] = mp(i - 1, j - 1);
      }
    }
  }

  int ans = dp[s.size()][t.size()];
  int spos = s.size(), tpos = t.size();
  while (ans) {
    pair<int, int> anc = p[spos][tpos];
    if (dp[spos][tpos] == dp[anc.first][anc.second] + 1) {
      --ans;
      s_positions->push_back(spos - 1);
      t_positions->push_back(tpos - 1);
    }

    spos = anc.first;
    tpos = anc.second;
  }

  reverse(s_positions->begin(), s_positions->end());
  reverse(t_positions->begin(), t_positions->end());
}

string MergeStrings(const string& s, const string& t) {
  vector<int> s_positions;
  vector<int> t_positions;
  LCS(s, t, &s_positions, &t_positions);
  int s_ptr = 0;
  int t_ptr = 0;
  string result;
  for (int i = 0; i < (int)s_positions.size(); ++i) {
    while (s_ptr < s_positions[i]) {
      result += s[s_ptr];
      ++s_ptr;
    }
    while (t_ptr < t_positions[i]) {
      result += t[t_ptr];
      ++t_ptr;
    }
    result += s[s_ptr];
    ++s_ptr;
    ++t_ptr;
  }
  while (s_ptr < s.size()) {
    result += s[s_ptr];
    ++s_ptr;
  }
  while (t_ptr < t.size()) {
    result += t[t_ptr];
    ++t_ptr;
  }
  return result;
}

string Solve(const string& S) {
  const int length_of_S = S.size();
  vector<int> last_occurence(kAlphabetSize, -1);
  vector<int> next_occurence(length_of_S, -1);
  for (int i = 0; i < (int)length_of_S; ++i) {
    if (last_occurence[S[i]] != -1) {
      next_occurence[last_occurence[S[i]]] = i;
    }
    last_occurence[S[i]] = i;
  }
  // Case 1: odd number of symbols in palindrom.
  vector<int> v(length_of_S, 0);
  vector<int> next(length_of_S);
  int min_length_odd = 2 * length_of_S + 1;
  int min_center_position_odd = -1;
  for (int center = 0; center < length_of_S; ++center) {
    int cur_length;
    if (center + 1 == length_of_S) {
      cur_length = 2 * length_of_S - 1;
    } else {
      cur_length = 1 +
          2 * (center + (length_of_S - center - 1 - v[center + 1]));
    }
    if (cur_length < min_length_odd) {
      min_length_odd = cur_length;
      min_center_position_odd = center;
    }

    next.assign(length_of_S, 0);

    int next_center_pos = next_occurence[center];
    for (int i = center + 1; i < length_of_S; ++i) {
      while (next_center_pos != -1 && next_center_pos < i) {
        next_center_pos = next_occurence[next_center_pos];
      }
      if (next_center_pos == -1) {
        next[i] = v[i];
      } else {
        if (next_center_pos + 1 < length_of_S) {
          next[i] = max(1 + v[next_center_pos + 1], v[i]);
        } else {
          next[i] = max(1, v[i]);
        }
      }
    }

    v.swap(next);
  }

  // Case 2: even number of symbols in palindrom.
  v.assign(length_of_S, 0);
  int last_zero_pos = last_occurence[S[0]];
  for (int i = last_zero_pos; i > 0; --i) {
    v[i] = 1;
  }

  int min_length_even = 2 * length_of_S + 1;
  int min_center_position_even = -1;
  for (int center = 0; center < length_of_S; ++center) {
    int cur_length;
    if (center + 1 == length_of_S) {
      cur_length = 2 * length_of_S;
    } else {
      cur_length = 2 *
          (center + 1 + (length_of_S - center - 1 - v[center + 1]));
    }

    if (cur_length < min_length_even) {
      min_length_even = cur_length;
      min_center_position_even = center;
    }

    next.assign(length_of_S, 0);


    int next_center_pos;
    if (center + 1 < length_of_S) {
      next_center_pos = next_occurence[center + 1];
    }
    for (int i = center + 2; i < length_of_S; ++i) {
      while (next_center_pos != -1 && next_center_pos < i) {
        next_center_pos = next_occurence[next_center_pos];
      }
      if (next_center_pos == -1) {
        next[i] = v[i];
      } else {
        if (next_center_pos + 1 < length_of_S) {
          next[i] = max(1 + v[next_center_pos + 1], v[i]);
        } else {
          next[i] = max(1, v[i]);
        }
      }
    }

    v.swap(next);
  } 
  if (min_length_odd < min_length_even) {
    string left = S.substr(0, min_center_position_odd);
    reverse(left.begin(), left.end());
    string right = S.substr(min_center_position_odd + 1);
    string right_palindrom = MergeStrings(left, right);
    string left_palindrom = right_palindrom;
    reverse(left_palindrom.begin(), left_palindrom.end());
    string ans = left_palindrom + S[min_center_position_odd] + right_palindrom;
    assert(ans.size() == min_length_odd);
    return ans;
  } else {
    string left = S.substr(0, min_center_position_even + 1);
    reverse(left.begin(), left.end());
    string right = S.substr(min_center_position_even + 1);
    string right_palindrom = MergeStrings(left, right);
    string left_palindrom = right_palindrom;
    reverse(left_palindrom.begin(), left_palindrom.end());
    string ans = left_palindrom + right_palindrom;
    assert(ans.size() == min_length_even);
    return ans;
  }
}

void Output(const string& S, const string& palindrom) {
  cout << (int)palindrom.size() - (int)S.size() << endl;
  cout << palindrom << endl;
}

int main() {
  const string S = Input();
  const string palindrom = Solve(S);
  Output(S, palindrom);
  vector<int> ns, ps;
  LCS(S, palindrom, &ns, &ps);
  assert(ns.size() == S.size());
  assert(IsPalindrom(palindrom));
}
