#include <algorithm>
#include <cassert>
#include <iostream>
#include <string>
#include <vector>
#include <cmath>
using namespace std;

#define mp make_pair

static const int kAlphabetSize = 300;

bool IsPalindrom(const string& s) {
  for (int i = 0; i < (int)s.size(); ++i) {
    if (s[i] != s[(int)s.size() - i - 1]) {
      return false;
    }
  }
  return true;
}

string Input() {
  string S;
  cin >> S;
  return S;
}

void LCS(const string& s, const string& t,
         vector<int>* s_positions, vector<int>* t_positions) {
  const int K = (int)max(sqrt(1.0 * s.size()), 1.00001);

  vector<vector<int> > column_history;
  vector<int> column_number;

  vector<int> prev(t.size() + 1, 0);
  vector<int> cur(t.size() + 1, 0);

  column_history.push_back(cur);
  column_number.push_back(0);

  for (int i = 1; i <= (int)s.size(); ++i) {
    prev = cur;
    cur.assign(t.size() + 1, 0);
    for (int j = 1; j <= (int)t.size(); ++j) {
      if (prev[j] > cur[j - 1]) {
        cur[j] = prev[j];
      } else {
        cur[j] = cur[j - 1];
      }

      if (s[i - 1] == t[j - 1] && cur[j] < prev[j - 1] + 1) {
        cur[j] = prev[j - 1] + 1;
      }
    }

    if (i % K == 0) {
      column_history.push_back(cur);
      column_number.push_back(i);
    }
  }

  if (s.size() % K != 0) {
    column_history.push_back(cur);
    column_number.push_back(s.size());
  }

  int tpos = t.size();
  vector<vector<int> > dp(K + 5, vector<int>(t.size() + 1));
  vector<vector<pair<int, int> > > p(
      K + 5, vector<pair<int, int> >(t.size() + 1,  mp(-1, -1)));
  for (int i = (int)column_history.size() - 1; i > 0; --i) {
    const vector<int> last = column_history[i];
    const vector<int> first = column_history[i - 1];

    for (int j = 0; j < (int)dp.size(); ++j) {
      for (int k = 0; k < (int)dp[j].size(); ++k) {
        dp[j][k] = 0;
        p[j][k] = mp(-1, -1);
      }
    }

    int num_col = column_number[i] - column_number[i - 1];

    dp[0] = first;
    for (int j = 1, r = column_number[i - 1] + 1; j <= num_col; ++j, ++r) {
      for (int k = 1; k <= t.size(); ++k) {
        if (dp[j - 1][k] > dp[j][k - 1]) {
          dp[j][k] = dp[j - 1][k];
          p[j][k] = mp(j - 1, k);
        } else {
          dp[j][k] = dp[j][k - 1];
          p[j][k] = mp(j, k - 1);
        }

        if (s[r - 1] == t[k - 1] && dp[j][k] < dp[j - 1][k - 1] + 1) {
          dp[j][k] = dp[j - 1][k - 1] + 1;
          p[j][k] = mp(j - 1, k - 1);
        }
      }
    }

    int spos = num_col;
    while (spos > 0 && tpos > 0) {
      pair<int, int> anc = p[spos][tpos];
      if (dp[spos][tpos] == dp[anc.first][anc.second] + 1) {
        s_positions->push_back(spos + column_number[i - 1] - 1);
        t_positions->push_back(tpos - 1);
      }

      spos = anc.first;
      tpos = anc.second;
    }
  }
  reverse(s_positions->begin(), s_positions->end());
  reverse(t_positions->begin(), t_positions->end());
}

string MergeStrings(const string& s, const string& t) {
  vector<int> s_positions;
  vector<int> t_positions;
  LCS(s, t, &s_positions, &t_positions);
  int s_ptr = 0;
  int t_ptr = 0;
  string result;
  for (int i = 0; i < (int)s_positions.size(); ++i) {
    while (s_ptr < s_positions[i]) {
      result += s[s_ptr];
      ++s_ptr;
    }
    while (t_ptr < t_positions[i]) {
      result += t[t_ptr];
      ++t_ptr;
    }
    result += s[s_ptr];
    ++s_ptr;
    ++t_ptr;
  }
  while (s_ptr < s.size()) {
    result += s[s_ptr];
    ++s_ptr;
  }
  while (t_ptr < t.size()) {
    result += t[t_ptr];
    ++t_ptr;
  }
  return result;
}

string Solve(const string& S) {
  const int length_of_S = S.size();
  vector<int> last_occurence(kAlphabetSize, -1);
  vector<int> next_occurence(length_of_S, -1);
  for (int i = 0; i < (int)length_of_S; ++i) {
    if (last_occurence[S[i]] != -1) {
      next_occurence[last_occurence[S[i]]] = i;
    }
    last_occurence[S[i]] = i;
  }
  // Case 1: odd number of symbols in palindrom.
  vector<int> v(length_of_S, 0);
  vector<int> next(length_of_S);
  int min_length_odd = 2 * length_of_S + 1;
  int min_center_position_odd = -1;
  for (int center = 0; center < length_of_S; ++center) {
    int cur_length;
    if (center + 1 == length_of_S) {
      cur_length = 2 * length_of_S - 1;
    } else {
      cur_length = 1 +
          2 * (center + (length_of_S - center - 1 - v[center + 1]));
    }
    if (cur_length < min_length_odd) {
      min_length_odd = cur_length;
      min_center_position_odd = center;
    }

    next.assign(length_of_S, 0);

    int next_center_pos = next_occurence[center];
    for (int i = center + 1; i < length_of_S; ++i) {
      while (next_center_pos != -1 && next_center_pos < i) {
        next_center_pos = next_occurence[next_center_pos];
      }
      if (next_center_pos == -1) {
        next[i] = v[i];
      } else {
        if (next_center_pos + 1 < length_of_S) {
          next[i] = max(1 + v[next_center_pos + 1], v[i]);
        } else {
          next[i] = max(1, v[i]);
        }
      }
    }

    v.swap(next);
  }

  // Case 2: even number of symbols in palindrom.
  v.assign(length_of_S, 0);
  int last_zero_pos = last_occurence[S[0]];
  for (int i = last_zero_pos; i > 0; --i) {
    v[i] = 1;
  }

  int min_length_even = 2 * length_of_S + 1;
  int min_center_position_even = -1;
  for (int center = 0; center < length_of_S; ++center) {
    int cur_length;
    if (center + 1 == length_of_S) {
      cur_length = 2 * length_of_S;
    } else {
      cur_length = 2 *
          (center + 1 + (length_of_S - center - 1 - v[center + 1]));
    }

    if (cur_length < min_length_even) {
      min_length_even = cur_length;
      min_center_position_even = center;
    }

    next.assign(length_of_S, 0);


    int next_center_pos;
    if (center + 1 < length_of_S) {
      next_center_pos = next_occurence[center + 1];
    }
    for (int i = center + 2; i < length_of_S; ++i) {
      while (next_center_pos != -1 && next_center_pos < i) {
        next_center_pos = next_occurence[next_center_pos];
      }
      if (next_center_pos == -1) {
        next[i] = v[i];
      } else {
        if (next_center_pos + 1 < length_of_S) {
          next[i] = max(1 + v[next_center_pos + 1], v[i]);
        } else {
          next[i] = max(1, v[i]);
        }
      }
    }

    v.swap(next);
  } 
  if (min_length_odd < min_length_even) {
    string left = S.substr(0, min_center_position_odd);
    reverse(left.begin(), left.end());
    string right = S.substr(min_center_position_odd + 1);
    string right_palindrom = MergeStrings(left, right);
    string left_palindrom = right_palindrom;
    reverse(left_palindrom.begin(), left_palindrom.end());
    string ans = left_palindrom + S[min_center_position_odd] + right_palindrom;
    assert(ans.size() == min_length_odd);
    return ans;
  } else {
    string left = S.substr(0, min_center_position_even + 1);
    reverse(left.begin(), left.end());
    string right = S.substr(min_center_position_even + 1);
    string right_palindrom = MergeStrings(left, right);
    string left_palindrom = right_palindrom;
    reverse(left_palindrom.begin(), left_palindrom.end());
    string ans = left_palindrom + right_palindrom;
    assert(ans.size() == min_length_even);
    return ans;
  }
}

void Output(const string& S, const string& palindrom) {
  cout << (int)palindrom.size() - (int)S.size() << endl;
  cout << palindrom << endl;
}

int main() {
  const string S = Input();
  const string palindrom = Solve(S);
  Output(S, palindrom);
}
