#include <algorithm>
#include <cassert>
#include <iostream>
#include <string>
#include <vector>
#include <utility>
using namespace std;

#define mp make_pair

static const int kAlphabetSize = 300;

bool IsPalindrom(const string& s) {
  for (int i = 0; i < (int)s.size(); ++i) {
    if (s[i] != s[(int)s.size() - i - 1]) {
      return false;
    }
  }
  return true;
}

string Input() {
  string S;
  cin >> S;
  return S;
}

void LCS(const string& s, const string& t,
         vector<int>* s_positions, vector<int>* t_positions) {
  vector<vector<int> > dp(s.size() + 1, vector<int>(t.size() + 1, 0));
  vector<vector<pair<int, int> > > p(
      s.size() + 1, vector<pair<int, int> >(t.size() + 1, mp(-1, -1)));

  for (int i = 1; i <= (int)s.size(); ++i) {
    for (int j = 1; j <= (int)t.size(); ++j) {
      if (dp[i - 1][j] > dp[i][j - 1]) {
        dp[i][j] = dp[i - 1][j];
        p[i][j] = mp(i - 1, j);
      } else {
        dp[i][j] = dp[i][j - 1];
        p[i][j] = mp(i, j - 1);
      }

      if (s[i - 1] == t[j - 1] && dp[i][j] < dp[i - 1][j - 1] + 1) {
        dp[i][j] = dp[i - 1][j - 1] + 1;
        p[i][j] = mp(i - 1, j - 1);
      }
    }
  }

  int ans = dp[s.size()][t.size()];
  int spos = s.size(), tpos = t.size();
  while (ans) {
    pair<int, int> anc = p[spos][tpos];
    if (dp[spos][tpos] == dp[anc.first][anc.second] + 1) {
      --ans;
      s_positions->push_back(spos - 1);
      t_positions->push_back(tpos - 1);
    }

    spos = anc.first;
    tpos = anc.second;
  }

  reverse(s_positions->begin(), s_positions->end());
  reverse(t_positions->begin(), t_positions->end());
}

string MergeStrings(const string& s, const string& t) {
  vector<int> s_positions;
  vector<int> t_positions;
  LCS(s, t, &s_positions, &t_positions);
  int s_ptr = 0;
  int t_ptr = 0;
  string result;
  for (int i = 0; i < (int)s_positions.size(); ++i) {
    while (s_ptr < s_positions[i]) {
      result += s[s_ptr];
      ++s_ptr;
    }
    while (t_ptr < t_positions[i]) {
      result += t[t_ptr];
      ++t_ptr;
    }
    result += s[s_ptr];
    ++s_ptr;
    ++t_ptr;
  }
  while (s_ptr < s.size()) {
    result += s[s_ptr];
    ++s_ptr;
  }
  while (t_ptr < t.size()) {
    result += t[t_ptr];
    ++t_ptr;
  }
  return result;
}

string BuildEvenLengthPalindrom(const string& S) {
  string allS = S;
  reverse(allS.begin(), allS.end());

  string res = S + allS;

  for (int left_length = 1; left_length < (int)S.size(); ++left_length) {
    string left = S.substr(0, left_length);
    string right = S.substr(left_length);

    reverse(left.begin(), left.end());
    string right_part = MergeStrings(left, right);
    string left_part = right_part;
    reverse(left_part.begin(), left_part.end());

    string palindrom = left_part + right_part;
    if (palindrom.size() < res.size()) {
      res = palindrom;
    }
  }
  return res;
}

string BuildOddLengthPalindrom(const string& S) {
  string res;

  for (int center = 0; center < (int)S.size(); ++center) {
    string left = S.substr(0, center);
    string right = S.substr(center + 1);

    reverse(left.begin(), left.end());
    string right_part = MergeStrings(left, right);
    string left_part = right_part;
    reverse(left_part.begin(), left_part.end());

    string palindrom = left_part + S[center] + right_part;

    if (res.size() == 0 || palindrom.size() < res.size()) {
      res = palindrom;
    }
  }
  return res;
}

string Solve(const string& S) {
  string even_length_palindrom = BuildEvenLengthPalindrom(S);
  string odd_length_palindrom = BuildOddLengthPalindrom(S);

  if (even_length_palindrom.size() < odd_length_palindrom.size()) {
    return even_length_palindrom;
  } else {
    return odd_length_palindrom;
  }
}

void Output(const string& S, const string& palindrom) {
  cout << (int)palindrom.size() - (int)S.size() << endl;
  cout << palindrom << endl;
}

int main() {
  const string S = Input();
  const string palindrom = Solve(S);
  Output(S, palindrom);
  // CHECK
  /*
  vector<int> ns, ps;
  LCS(S, palindrom, &ns, &ps);
  assert(ns.size() == S.size());
  assert(IsPalindrom(palindrom));
  */
}
