#include <algorithm>
#include <cassert>
#include <iostream>
#include <string>
#include <vector>
using namespace std;

static const int kAlphabetSize = 300;

bool IsPalindrom(const string& s) {
  for (int i = 0; i < (int)s.size(); ++i) {
    if (s[i] != s[(int)s.size() - i - 1]) {
      return false;
    }
  }
  return true;
}

string Input() {
  string S;
  cin >> S;
  return S;
}

void LCS(const string& s, const string& t,
         vector<int>* s_positions, vector<int>* t_positions) {
  if (s.empty() || t.empty()) {
    return;
  }
  vector<int> v(t.size(), 0);
  vector<int> next(t.size(), 0);
  vector<int> start(t.size(), 0);
  vector<int> max_start(t.size() + 1, -1);
  for (int i = (int)s.size() - 1; i >= 0; --i) {
    next.assign(t.size(), 0);
    for (int j = (int)t.size() - 1; j >= 0; --j) {
      if (s[i] == t[j]) {
        if (j + 1 == (int)t.size()) {
          if (v[j] == 0) {
            start[j] = i;
          }
          next[j] = 1;
        } else {
          if (1 + v[j + 1] > v[j]) {
            start[j] = i;
            next[j] = 1 + v[j + 1];
          } else {
            next[j] = v[j];
          }
        }
      } else {
        if (j + 1 == (int)t.size()) {
          next[j] = v[j];
        } else {
          if (next[j + 1] > v[j]) {
            start[j] = start[j + 1];
            next[j] = next[j + 1];
          } else {
            next[j] = v[j];
          }
        }
      }
      max_start[next[j]] = max(max_start[next[j]], start[j]);
    }
    v.swap(next);
  }

  for (int i = v[0]; i > 0; --i) {
    s_positions->push_back(max_start[i]);
  }
  int p = 0;
  for (int i = 0; i < (int)s_positions->size(); ++i) {
    while (t[p] != s[(*s_positions)[i]]) {
      ++p;
    }
    t_positions->push_back(p);
    ++p;
  }
}

string MergeStrings(const string& s, const string& t) {
  vector<int> s_positions;
  vector<int> t_positions;
  LCS(s, t, &s_positions, &t_positions);
  int s_ptr = 0;
  int t_ptr = 0;
  string result;
  for (int i = 0; i < (int)s_positions.size(); ++i) {
    while (s_ptr < s_positions[i]) {
      result += s[s_ptr];
      ++s_ptr;
    }
    while (t_ptr < t_positions[i]) {
      result += t[t_ptr];
      ++t_ptr;
    }
    result += s[s_ptr];
    ++s_ptr;
    ++t_ptr;
  }
  while (s_ptr < s.size()) {
    result += s[s_ptr];
    ++s_ptr;
  }
  while (t_ptr < t.size()) {
    result += t[t_ptr];
    ++t_ptr;
  }
  return result;
}

string Solve(const string& S) {
  const int length_of_S = S.size();
  vector<int> last_occurence(kAlphabetSize, -1);
  vector<int> next_occurence(length_of_S, -1);
  for (int i = 0; i < (int)length_of_S; ++i) {
    if (last_occurence[S[i]] != -1) {
      next_occurence[last_occurence[S[i]]] = i;
    }
    last_occurence[S[i]] = i;
  }
  // Case 1: odd number of symbols in palindrom.
  vector<int> v(length_of_S, 0);
  vector<int> next(length_of_S);
  int min_length_odd = 2 * length_of_S + 1;
  int min_center_position_odd = -1;
  for (int center = 0; center < length_of_S; ++center) {
    int cur_length;
    if (center + 1 == length_of_S) {
      cur_length = 2 * length_of_S - 1;
    } else {
      cur_length = 1 +
          2 * (center + (length_of_S - center - 1 - v[center + 1]));
    }
    if (cur_length < min_length_odd) {
      min_length_odd = cur_length;
      min_center_position_odd = center;
    }

    next.assign(length_of_S, 0);

    int next_center_pos = next_occurence[center];
    for (int i = center + 1; i < length_of_S; ++i) {
      while (next_center_pos != -1 && next_center_pos < i) {
        next_center_pos = next_occurence[next_center_pos];
      }
      if (next_center_pos == -1) {
        next[i] = v[i];
      } else {
        if (next_center_pos + 1 < length_of_S) {
          next[i] = max(1 + v[next_center_pos + 1], v[i]);
        } else {
          next[i] = max(1, v[i]);
        }
      }
    }

    v.swap(next);
  }

  // Case 2: even number of symbols in palindrom.
  v.assign(length_of_S, 0);
  int last_zero_pos = last_occurence[S[0]];
  for (int i = last_zero_pos; i > 0; --i) {
    v[i] = 1;
  }

  int min_length_even = 2 * length_of_S + 1;
  int min_center_position_even = -1;
  for (int center = 0; center < length_of_S; ++center) {
    int cur_length;
    if (center + 1 == length_of_S) {
      cur_length = 2 * length_of_S;
    } else {
      cur_length = 2 *
          (center + 1 + (length_of_S - center - 1 - v[center + 1]));
    }

    if (cur_length < min_length_even) {
      min_length_even = cur_length;
      min_center_position_even = center;
    }

    next.assign(length_of_S, 0);


    int next_center_pos;
    if (center + 1 < length_of_S) {
      next_center_pos = next_occurence[center + 1];
    }
    for (int i = center + 2; i < length_of_S; ++i) {
      while (next_center_pos != -1 && next_center_pos < i) {
        next_center_pos = next_occurence[next_center_pos];
      }
      if (next_center_pos == -1) {
        next[i] = v[i];
      } else {
        if (next_center_pos + 1 < length_of_S) {
          next[i] = max(1 + v[next_center_pos + 1], v[i]);
        } else {
          next[i] = max(1, v[i]);
        }
      }
    }

    v.swap(next);
  } 
  if (min_length_odd < min_length_even) {
    string left = S.substr(0, min_center_position_odd);
    reverse(left.begin(), left.end());
    string right = S.substr(min_center_position_odd + 1);
    string right_palindrom = MergeStrings(left, right);
    string left_palindrom = right_palindrom;
    reverse(left_palindrom.begin(), left_palindrom.end());
    string ans = left_palindrom + S[min_center_position_odd] + right_palindrom;
    assert(ans.size() == min_length_odd);
    return ans;
  } else {
    string left = S.substr(0, min_center_position_even + 1);
    reverse(left.begin(), left.end());
    string right = S.substr(min_center_position_even + 1);
    string right_palindrom = MergeStrings(left, right);
    string left_palindrom = right_palindrom;
    reverse(left_palindrom.begin(), left_palindrom.end());
    string ans = left_palindrom + right_palindrom;
    assert(ans.size() == min_length_even);
    return ans;
  }
}

void Output(const string& S, const string& palindrom) {
  cout << (int)palindrom.size() - (int)S.size() << endl;
  cout << palindrom << endl;
}

int main() {
  const string S = Input();
  const string palindrom = Solve(S);
  Output(S, palindrom);
  vector<int> ns, ps;
  LCS(S, palindrom, &ns, &ps);
  assert(ns.size() == S.size());
  assert(IsPalindrom(palindrom));
}
