#include "testlib.h"

using namespace std;

// reading unsigned long long in testlib style
unsigned long long readUnsignedLong(InStream& in, unsigned long long minv, 
                                    unsigned long long maxv, 
                                    const std::string& variableName = "");
 
static inline unsigned long long stringToUnsignedLongLong(InStream& in,
                                                          const char* buffer) {
  size_t length = strlen(buffer);
  pattern digits("[0-9]+");
 
  if (!digits.matches(buffer)) {
    in.quit(_pe, ("Expected integer, but \"" + __testlib_part(buffer) +
      "\" doesn't correspond to pattern \"" + digits.src() + "\"").c_str());
  }
 
  if ((length > 1 && buffer[0] == '0') || length > 20 ||
      (length == 20 && strcmp(buffer, "18446744073709551615") == 1)) {
    in.quit(_pe, ("Expected integer, but \"" + __testlib_part(buffer) +
      "\" found").c_str());
  }
  
  unsigned long long retval;
  std::istringstream s(buffer);
  s >> retval;
  return retval;
}
 
unsigned long long readUnsignedLong(InStream& in, unsigned long long minv,
                                    unsigned long long maxv,
                                    const std::string& variableName) {
  if (!in.strict && in.seekEof()) {
    quit(_unexpected_eof, "Unexpected end of file - int64 expected");
  }
 
  std::string number;
  in.readWordTo(number);
  unsigned long long result = stringToUnsignedLongLong(in, number.c_str());
 
  if (result < minv || result > maxv) {
    if (variableName.empty()) {
      quit(_wa, ("Integer " + vtos(result) + " violates the range [" +
        vtos(minv) + ", " + vtos(maxv) + "]").c_str());
    } else {
      quit(_wa, ("Integer parameter [name=" + variableName + "] equals to " +
        vtos(result) + ", violates the range [" + vtos(minv) + ", " +
        vtos(maxv) + "]").c_str());
    }
  }

  return result;
}

// checker itself
unsigned long long n;

const unsigned long long MAXN = static_cast<unsigned long long>(-1);

int read_answer(InStream &stream) {
    int k = stream.readInt();
    unsigned long long curr_n = 2;
    for (int i = 0; i < k; ++i) {
      unsigned long long a = readUnsignedLong(stream, 1, n);
      curr_n += (curr_n - 1) * a;
    }
    if (curr_n != n) {
      stream.quitf(_wa, "Final number of people in queue is %s, but %s expected", 
        vtos(curr_n).c_str(), vtos(n).c_str());
    }
    return k;
}

int main(int argc, char **argv) {
    registerTestlibCmd(argc, argv);

    n = readUnsignedLong(inf, 3, MAXN, "n");

    int jans = read_answer(ans);
    int pans = read_answer(ouf);
    if (jans > pans) {
      quitf(_wa, "jury has the better k: jans = %d, pans = %d\n", jans, pans);
    } else if (jans == pans) {
      quitf(_ok, "k = %d\n", pans);
    } else { // (jans < pans)
      quitf(_fail, ":( participant has the better k: jans = %d, pans = %d\n", jans, pans);
    }
}

