import sys
import math
import random

def is_prime(n):
    assert n >= 2
    
    if n == 2:
        return True
    
    if n % 2 == 0:
        return False
    
    s = 0
    d = n - 1
    while True:
        quotient, remainder = divmod(d, 2)
        if remainder == 1:
            break
        s += 1
        d = quotient
    assert 2 ** s * d == n - 1
 
    # test the base a to see whether it is a witness for the compositeness of n
    def try_composite(a):
        if pow(a, d, n) == 1:
            return False
        for i in range(s):
            if pow(a, 2 ** i * d, n) == n - 1:
                return False
        return True # n is definitely composite
 
    for a in [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37]: # n < 2 ^ 64, that is enough
        if try_composite(a):
            return False

    return True

    
def gcd(a, b):
    return a if b == 0 else gcd(b, a % b)

    
def find_prime_divisors_leman(n):    
    if is_prime(n):
        return [n]
    
    divisors = []
    sqrt3_n_up = math.ceil(n ** (1 / 3)) + 1
    
    # finding divisors <= n ^ (1 / 3)
    m = 2    
    while m * m <= n and m < sqrt3_n_up:
        while n % m == 0:
            divisors.append(m)
            n //= m
        m += 1
    if m * m > n:
        if n != 1:
            divisors.append(n)
        return divisors
        
    # either n is prime or n = p * q,  n ^ (1 / 3) < p <= q < n ^ (2 / 3)
    if is_prime(n):
        divisors.append(n)
        return divisors
        
    sqrt6_n = n ** (1 / 6)
    for k in range(1, sqrt3_n_up):
        max_d = math.ceil( sqrt6_n / (4 * k ** (1 / 2))) + 2
        s = 4 * k * n
        t = math.floor(s ** (1 / 2))
        if t * t < s:
            t += 1
        assert (t - 1) ** 2 < s and t ** 2 >= s
        
        diff = t ** 2 - s
        for d in range(max_d):
            a = t + d            
            b = round(diff ** (1 / 2))
            if b * b == diff:
                # a ^ 2 = b ^ 2 mod n
                d1 = gcd(a + b, n)
                d2 = gcd(a - b, n)
                if 1 < d1 < n or 1 < d2 < n:
                    p = d1 if 1 < d1 < n else d2
                    divisors.append(p)
                    divisors.append(n // p)
                    return divisors
            diff += 2 * a + 1
            
    # n is prime
    divisors.append(n)
    return divisors
    
def upper_prime(n):
    for i in range(10 ** 10):
        if is_prime(n + i):
            return n + i

def lower_prime(n):
    for i in range(10 ** 10):
        if is_prime(n - i):
            return n - i
    
#infile = "E:\\Users\\Nikita\\Documents\\FactorizationQueue\\output.txt"
#outfile = "E:\\Users\\Nikita\\Documents\\FactorizationQueue\\input.txt"
n = 2 ** 64 - 1
start = 2642245 + int(random.random() * 100000)
x = upper_prime(start)        
y = lower_prime(n // x)
print (x * y + 1)

#for i in range(1000):    
#    x = upper_prime(start)        
#    y = lower_prime(n // x)
#    print (x, y)
    
#    d = sorted(find_prime_divisors_leman(x * y))
#    if len(d) < 2 or d[0] != x or d[1] != y:
#        print(x, y, x * y, d)
        
#    start = x + 1    
