#include "testlib.h"

// reading unsigned long long in testlib style
unsigned long long readUnsignedLong(InStream& in, unsigned long long minv, 
                                    unsigned long long maxv, 
                                    const std::string& variableName = "");
 
static inline unsigned long long stringToUnsignedLongLong(InStream& in,
                                                          const char* buffer) {
  size_t length = strlen(buffer);
  pattern digits("[0-9]+");
 
  if (!digits.matches(buffer)) {
    in.quit(_pe, ("Expected integer, but \"" + __testlib_part(buffer) +
      "\" doesn't correspond to pattern \"" + digits.src() + "\"").c_str());
  }
 
  if ((length > 1 && buffer[0] == '0') || length > 20 ||
      (length == 20 && strcmp(buffer, "18446744073709551615") == 1)) {
    in.quit(_pe, ("Expected integer, but \"" + __testlib_part(buffer) +
      "\" found").c_str());
  }
  
  unsigned long long retval;
  std::istringstream s(buffer);
  s >> retval;
  return retval;
}
 
unsigned long long readUnsignedLong(InStream& in, unsigned long long minv,
                                    unsigned long long maxv,
                                    const std::string& variableName) {
  if (!in.strict && in.seekEof()) {
    quit(_unexpected_eof, "Unexpected end of file - int64 expected");
  }
 
  std::string number;
  in.readWordTo(number);
  unsigned long long result = stringToUnsignedLongLong(in, number.c_str());
 
  if (result < minv || result > maxv) {
    if (variableName.empty()) {
      quit(_wa, ("Integer " + vtos(result) + " violates the range [" +
        vtos(minv) + ", " + vtos(maxv) + "]").c_str());
    } else {
      quit(_wa, ("Integer parameter [name=" + variableName + "] equals to " +
        vtos(result) + ", violates the range [" + vtos(minv) + ", " +
        vtos(maxv) + "]").c_str());
    }
  }

  return result;
}

// validator itself
const unsigned long long MAXN[] = {
    static_cast<unsigned long long>(-1),
    20, 
    100, 
    10 * 1000, 
    1000 * 1000,
    static_cast<unsigned long long>(1000) * 1000 * 1000 * 1000,
    static_cast<unsigned long long>(-1)
};

int main(int argc, char **argv) {
    registerValidation(argc, argv);
    int group = atoi(validator.group().data());
    ensuref(0 <= group && group <= 6, "Incorrect test group");
    unsigned long long n = readUnsignedLong(inf, 3, MAXN[group]);
    inf.readEoln();
    inf.readEof();
    return 0;    
}

