#include <iostream>
#include <vector>
#include <cmath>
#include <array>

using namespace std;

unsigned long long gcd(unsigned long long a, unsigned long long b) {
    return (b == 0) ? a : gcd(b, a % b);
}

typedef array<unsigned, 4> Int128;
const unsigned long long BASE = (unsigned long long)1 << 32;

Int128 ULLtoInt128(unsigned long long a) {
    Int128 x;
    x[0] = a % BASE;
    x[1] = a / BASE;
    return x;
}

Int128 multiplyLongLong(unsigned long long a, unsigned long long b) {
    Int128 x = ULLtoInt128(a);
    Int128 y = ULLtoInt128(b);
    Int128 result;
    unsigned long long z[4] = {0};
    for (int i = 0; i < 2; ++i)
        for (int j = 0; j < 2; ++j)
            z[i + j] += (unsigned long long)x[i] * y[j];
    for (int i = 0; i < 3; ++i) {
        result[i] = z[i] % BASE;
        z[i + 1] += z[i] / BASE;
    }
    return result;
}

// we presume a > b
Int128 subtractInt128(Int128 a, Int128 b) {
    Int128 result;
    unsigned long long debt = 0;
    for (int i = 0; i < 4; ++i) {
        if (a[i] < b[i] + debt) {
            result[i] = (a[i] + BASE) - (b[i] + debt);
            debt = 1;
        } else {
            result[i] = a[i] - (b[i] + debt);
            debt = 0;
        }
    }
    return result;
}

// we expect a to be short enough
unsigned long long Int128toULL(Int128 a) {
    return a[0] + a[1] * BASE;
}

// find r^2 - a * b (we presume r^2 > a * b)
unsigned long long find_diff(unsigned long long r, unsigned long long a,
                             unsigned long long b) {
    return Int128toULL(subtractInt128(multiplyLongLong(r, r),
                                      multiplyLongLong(a, b)));
}

vector<unsigned long long> find_prime_divisors_leman(unsigned long long n) {
    vector<unsigned long long> divisors;
    unsigned long long sqrt3_n_up = ceil(pow((long double)n, (long double)1.0 / 3)) + 2;
    
    // finding divisors <= n ^ (1 / 3)
    unsigned long long m = 2;
    while (m * m <= n && m < sqrt3_n_up) {
        while (n % m == 0) {
            divisors.push_back(m);
            n /= m;
        }
        ++m;
    }
    if (m * m > n) {
        if (n != 1) {
            divisors.push_back(n);
        }
        return divisors;
    }
        
    // either n is prime or n = p * q,  n ^ (1 / 3) < p <= q < n ^ (2 / 3)
    long double sqrt6_n = pow((long double)n, (long double)1.0 / 6);
    long double sqrt_n = sqrt((long double)n);
    for(int k = 1; k < sqrt3_n_up; ++k) {
        long double sqrt_k = sqrt((long double)k);
        unsigned long long sqrt_4nk = ceil(2 * sqrt_k * sqrt_n) + 2;
        
        // diff = ([sqrt(4nk)]^2 - 4nk)
        unsigned long long diff = find_diff(sqrt_4nk, 4 * k, n);
        while (diff >= 2 * sqrt_4nk - 1) {
            diff = diff - 2 * sqrt_4nk + 1;
            --sqrt_4nk;
        }
        int max_d = ceil(sqrt6_n / (4 * sqrt_k)) + 3;
        for (int d = 0; d < max_d; ++d) {
            unsigned long long a = sqrt_4nk + d;
            unsigned long long b = round(sqrt((long double)diff));
            if (b * b == diff) {
                // diff is a perfect square, a ^ 2 = b ^ 2 mod n
                unsigned long long d1 = gcd(a + b, n);
                unsigned long long d2 = gcd(a - b, n);
                if (1 < d1 && d1 < n || 1 < d2 && d2 < n) {
                    unsigned long long p = (1 < d1 && d1 < n) ? d1 : d2;
                    divisors.push_back(p);
                    divisors.push_back(n / p);
                    return divisors;
                }
            }
            diff += 2 * a + 1;
        }
    }

    // n is prime
    divisors.push_back(n);
    return divisors;
}

int main() {
    unsigned long long n;
    cin >> n;
    vector<unsigned long long> divisors = find_prime_divisors_leman(n - 1);
    cout << divisors.size() << endl;
    for (int i = 0; i < divisors.size(); ++i) {
        cout << ((i == 0) ? "" : " ") << divisors[i] - 1;
    }
    cout << endl;
    return 0;
}
