import random
import math


def gcd(a, b):
    return a if b == 0 else gcd(b, a % b)


def is_prime(n):
    assert n >= 2
    
    if n == 2:
        return True
    
    if n % 2 == 0:
        return False
    
    s = 0
    d = n - 1
    while True:
        quotient, remainder = divmod(d, 2)
        if remainder == 1:
            break
        s += 1
        d = quotient
    assert 2 ** s * d == n - 1
 
    # test the base a to see whether it is a witness for the compositeness of n
    def try_composite(a):
        if pow(a, d, n) == 1:
            return False
        for i in range(s):
            if pow(a, 2 ** i * d, n) == n - 1:
                return False
        return True # n is definitely composite
 
    for a in [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37]: # n < 2 ^ 64, that is enough
        if try_composite(a):
            return False

    return True


def rho_pollard(n, max_iter = 1000000):
    b0, i = random.randint(1, n - 1), 0
    b1 = (b0 ** 2 + 1) % n
    while (not 1 < gcd(n, abs(b1 - b0)) < n) and i < max_iter:
        b0 = (b0 ** 2 + 1) % n
        b1 = ((b1 ** 2 + 1) ** 2 + 1) % n
        i += 1
    return gcd(n, abs(b1 - b0))


def find_prime_divisors_simple(n):
    divisors = []    
    m = 2    
    while m * m <= n:
        while n % m == 0:
            divisors.append(m)
            n //= m
        m += 1
    if m * m > n:
        if n != 1:
            divisors.append(n)            
    return divisors


def find_prime_divisors_pollard_rho(n, max_starts = 5):
    if n == 1:
        return []
    
    if is_prime(n):
        return [n]
    
    if n < 1000000000:
        return find_prime_divisors_simple(n)
    
    for i in range(max_starts):
        d = rho_pollard(n)
        if 1 < d < n:
            break;
        
    if 1 < d < n:
        power = 0
        while n % d == 0:
            n //= d
            power += 1
            
        return (find_prime_divisors_pollard_rho(d) * power + 
                find_prime_divisors_pollard_rho(n))
    else:
        return [n]


random.seed(1543)
n = int(input())
prime_divisors = find_prime_divisors_pollard_rho(n - 1)
print(len(prime_divisors))
print(' '.join(map(lambda x: str(x - 1), prime_divisors)))
