#include <iostream>
#include <vector>

using namespace std;

enum {
  PRIMES_BORDER = 10000000
};

vector<unsigned> find_primes_eratosthenes_sieve(int prime_border) {
  vector<bool> is_prime(prime_border + 1, true);
  vector<unsigned> primes;
  for (int i = 2; i < is_prime.size(); ++i) {
    if (is_prime[i]) {
      primes.push_back(i);
      for (int k = i + i; k < is_prime.size(); k += i) {
        is_prime[k] = false;
      }
    }
  }
  return primes;
}

vector<unsigned long long> factorize_eratosthenes(unsigned long long n) {
  vector<unsigned> primes = find_primes_eratosthenes_sieve(PRIMES_BORDER);
  vector<unsigned long long> divisors;
  for (int p : primes) {
    while (n % p == 0) {
      n /= p;
      divisors.push_back(p);
    }
  }
  if (n > 1) {
    divisors.push_back(n);
  }
  return divisors;
}

int main() {
  unsigned long long n;
  cin >> n;
  vector<unsigned long long> divisors = factorize_eratosthenes(n - 1);
  cout << divisors.size() << endl;
  for (int i = 0; i < divisors.size(); ++i) {
    cout << ((i == 0) ? "" : " ") << divisors[i] - 1;
  }
  cout << endl;
  return 0;
}